"use strict";

var bookingForms = [],
    geocoder,
    directionsService,
    pricing_data;

function initForms() {
  if ( jQuery('.quickcab-booking-form').length == 0 ) {
    return;
  }

  // initialise google maps objects
  geocoder = new google.maps.Geocoder();
  directionsService = new google.maps.DirectionsService();

  // create form objects
  jQuery('.quickcab-booking-form').each(function() {
    var $form = jQuery(this);

    initialiseQuickcabBookingForms($form);
  });
}

function initialiseQuickcabBookingForms($form) {
  var formId = parseInt($form.attr('data-form-id'));
  var minDate = new Date();

  $form.find('.quickcab-form-departure-date').datepicker({
    dateFormat: 'dd/mm/yy',
    minDate: minDate,
    showOtherMonths: true,
    dayNamesMin: ['S', 'M', 'T', 'W', 'T', 'F', 'S'],
    beforeShow: function(input, instance) {
      jQuery(instance.dpDiv).attr('data-form-id', formId);
    },
    onSelect: function(date) {
      var splitDate = date.split('/').reverse();

      var returnMinDate = new Date(
        parseInt(splitDate[0]),
        parseInt(splitDate[1]) - 1,
        parseInt(splitDate[2]),
        0,
        0,
        0,
        0
      );

      $form.find('.quickcab-form-return-departure-date').datepicker('option', 'minDate', returnMinDate);
      jQuery(this).removeClass('quickcab-field-error');
    }
  });

  $form.find('.quickcab-form-return-departure-date').datepicker({
    dateFormat: 'dd/mm/yy',
    minDate: minDate,
    showOtherMonths: true,
    dayNamesMin: ['S', 'M', 'T', 'W', 'T', 'F', 'S'],
    onSelect: function(date) {
      jQuery(this).removeClass('quickcab-field-error');
    }
  });

  $form.find('.quickcab-form-direction').change(function() {
    if ( this.value == 'return' ) {
      $form.find('.quickcab-return-journey-options').show();
    } else {
      $form.find('.quickcab-return-journey-options').hide();
    }
  });

  $form.on('click', '.quickcab-vehicle-book-button', function(e) {
    e.preventDefault();
  
    var $this = jQuery(this);
  
    var vehicleID = parseInt($this.attr('data-vehicle-id'));
    var index = parseInt($this.attr('data-index'));
  
    $this.text($this.attr('data-disable-with'));
  
    $form.find('.quickcab-form-vehicle-id').val(vehicleID);
    $form.find('.quickcab-total').val(pricing_data[index].quote.total_breakdown.total.amount_unformatted);
    $form.submit();
  });

  $form.find('.quickcab-form-back-button').on('click', function(e) {
    e.preventDefault();

    var formId = parseInt($form.attr('data-form-id'));
    var formObject = bookingForms.find(function(form) {
      return form.formId == formId;
    });

    quickcabMoveToTab(formObject, 0);
  });

  $form.find('.quickcab-select-input').change(function(e) {
    if ( this.value.length > 0 ) {
      jQuery(this).removeClass('quickcab-field-error');
    }
  });

  $form.on('click', '.quickcab-add-waypoint', function(e) {
    e.preventDefault();

    var index = parseInt(jQuery(this).attr('data-insert-at'));
    var formId = parseInt($form.attr('data-form-id'));
    var formObject = bookingForms.find(function(form) {
      return form.formId == formId;
    });

    quickcabAddWaypointField($form, index);
    quickcabRecalculateWaypointIndices(formObject);
  });

  $form.on('click', '.quickcab-remove-waypoint', function(e) {
    e.preventDefault();

    var index = parseInt(jQuery(this).attr('data-remove-at'));
    var formId = parseInt($form.attr('data-form-id'));
    var formObject = bookingForms.find(function(form) {
      return form.formId == formId;
    });

    quickcabRemoveWaypointField($form, index);
    quickcabRecalculateWaypointIndices(formObject);
    quickcabCalculateRoute(formObject);
  });

  buildQuickcabFormObject($form);
}

function buildQuickcabFormObject($form) {
  var formId = parseInt($form.attr('data-form-id'));
  var $map = $form.find('.quickcab-form-map');

  var autocompleteOptions = {};

  if ( quickcab_js_settings.forms[formId].enforce_autocomplete_restriction ) {
    autocompleteOptions['componentRestrictions'] = {
      country: quickcab_js_settings.forms[formId].autocomplete_country_restriction
    };
  }

  // center map around starting location specified in settings
  var starting_location = quickcab_js_settings.forms[formId].map_starting_location;

  if ( starting_location ) {
    geocoder.geocode({ 'address': starting_location }, function(results, status) {
      if (status == google.maps.GeocoderStatus.OK) {
        var latLng = { lat: results[0].geometry.location.lat(), lng: results[0].geometry.location.lng() };

        // done here to ensure map doesn't render when location not known
        formObject.map = new google.maps.Map($map.get(0), {
          center: { lat: 51.507351, lng: -0.127758 },
          zoom: 8,
          zoomControl: quickcab_js_settings.forms[formId].map.enable_zoom_control,
          mapTypeControl: quickcab_js_settings.forms[formId].map.enable_map_type_control,
          streetViewControl: quickcab_js_settings.forms[formId].map.enable_street_view_control,
          fullscreenControl: quickcab_js_settings.forms[formId].map.enable_full_screen_control,
          gestureHandling: quickcab_js_settings.forms[formId].map.enable_pan_control ? 'auto' : 'none'
        });

        formObject.directionsDisplay = new google.maps.DirectionsRenderer({
          map: formObject.map,
          suppressMarkers: true,
          preserveViewport: true
        });

        formObject.map.setCenter(latLng);
        formObject.map.fitBounds(results[0].geometry.bounds);
      } else {
        formObject.map = new google.maps.Map($map.get(0), {
          center: { lat: 51.507351, lng: -0.127758 },
          zoom: 8
        });

        formObject.directionsDisplay = new google.maps.DirectionsRenderer({
          map: formObject.map,
          suppressMarkers: true,
          preserveViewport: true
        });
      }

      wpdk_do_action('quickcab_form_map_init', formObject);
    });
  }

  var formObject = {
    '$form': $form,
    formId: formId,
    autocompleteOptions: autocompleteOptions,
    origin: {
      autocomplete: null,
      marker: null,
      lat: null,
      lng: null,
      address: null
    },
    destination: {
      autocomplete: null,
      marker: null,
      lat: null,
      lng: null,
      address: null
    },
    waypoints: [],
    activeTabs: [true, false],
    activeTab: 0,
    routeStats: {
      totalDistance: null,
      totalDuration: null
    },
    showDistance: quickcab_js_settings.forms[formId].show_distance,
    showDuration: quickcab_js_settings.forms[formId].show_duration,
    requiredFields: [
      {
        "text": [
          'quickcab_form_departure_date_' + formId
        ],
        "google_autocomplete": [],
        "select": [
          'quickcab_form_departure_time_' + formId,
        ]
      },
    ]
  };

  initializeOriginAddressInput(formObject);
  initializeDestinationAddressInput(formObject);

  bookingForms.push(formObject);
}

function initializeOriginAddressInput(formObject) {
  var $startingAddressInput = formObject.$form.find('.quickcab-form-starting-address');

  if ( quickcab_js_settings.forms[formObject.formId].use_fixed_addresses_starting ) {
    // add field to custom error checking object
    formObject.requiredFields[0]['select'].push('quickcab_form_starting_address_' + formObject.formId);

    // if using fixed addreses for starting address
    $startingAddressInput.change(function() {
      // remove error on resubmit
      $startingAddressInput.removeClass('quickcab-field-error');

      formObject.origin.address = jQuery(this).val();

      geocoder.geocode({ 'address': $startingAddressInput.val() }, function(results, status) {
        if (status == google.maps.GeocoderStatus.OK) {
          quickcabHandleOriginGeocode(formObject, results);
        }
      });
    });
  } else {
    // user types address
    formObject.origin.autocomplete = new google.maps.places.Autocomplete(
      $startingAddressInput.get(0),
      formObject.autocompleteOptions
    );

    // add autocomplete field to custom error checking object
    formObject.requiredFields[0]['google_autocomplete'].push({
      "id": 'quickcab_form_starting_address_' + formObject.formId,
      "var": formObject.origin.autocomplete
    });

    // when a place has been selected, plot it on the map
    google.maps.event.addListener(formObject.origin.autocomplete, 'place_changed', function() {
      var startingGooglePlace = formObject.origin.autocomplete.getPlace();
      var addressValue = $startingAddressInput.val();

      if ( startingGooglePlace.hasOwnProperty('place_id') && startingGooglePlace.place_id.length > 0 ) {
        var startingGeocodeOptions = { 'placeId': startingGooglePlace.place_id };
      } else {
        var startingGeocodeOptions = { 'address': addressValue };
      }

      // remove any errors that might have been there from last submit
      $startingAddressInput.removeClass('quickcab-field-error');
      formObject.$form.find('.qc_form_warning_quickcab_form_starting_address_' + formObject.formId).slideUp();

      // add address to hidden input to be saved when order placed
      $startingAddressInput.siblings('.quickcab-hidden-autocomplete-field').val(addressValue);
      formObject.origin.address = addressValue;

      // geocode address to get coords
      geocoder.geocode(startingGeocodeOptions, function(results, status) {
        if (status == google.maps.GeocoderStatus.OK) {
          quickcabHandleOriginGeocode(formObject, results);
        }
      });
    });

    // when enter pressed do nothing
    google.maps.event.addDomListener($startingAddressInput.get(0), 'keydown', function(e) {
      if (e.keyCode == 13) {
        e.preventDefault();
      }
    });

    $startingAddressInput.on('input', function() {
      jQuery(this).siblings('.quickcab-hidden-autocomplete-field').val('');
    });
  }
}

function initializeDestinationAddressInput(formObject) {
  var $destinationAddressInput = formObject.$form.find('.quickcab-form-destination-address');

  if ( quickcab_js_settings.forms[formObject.formId].use_fixed_addresses_destination ) {
    formObject.requiredFields[0]['select'].push('quickcab_form_destination_address_' + formObject.formId);

    // if using fixed addreses for destination address
    $destinationAddressInput.change(function() {
      // remove error on resubmit
      $destinationAddressInput.removeClass('quickcab-field-error');

      formObject.destination.address = jQuery(this).val();

      geocoder.geocode({ 'address': $destinationAddressInput.val() }, function(results, status) {
        if (status == google.maps.GeocoderStatus.OK) {
          quickcabHandleDestinationGeocode(formObject, results);
        }
      });
    });
  } else {
    // user types address
    formObject.destination.autocomplete = new google.maps.places.Autocomplete(
      $destinationAddressInput.get(0),
      formObject.autocompleteOptions
    );

    // add autocomplete field to custom error checking object
    formObject.requiredFields[0]['google_autocomplete'].push({
      "id": 'quickcab_form_destination_address_' + formObject.formId,
      "var": formObject.destination.autocomplete
    });

    // when a place has been selected, plot it on the map
    google.maps.event.addListener(formObject.destination.autocomplete, 'place_changed', function() {
      var destinationGooglePlace = formObject.destination.autocomplete.getPlace();
      var addressValue = $destinationAddressInput.val();

      if ( destinationGooglePlace.hasOwnProperty('place_id') && destinationGooglePlace.place_id.length > 0 ) {
        var destinationGeocodeOptions = { 'placeId': destinationGooglePlace.place_id };
      } else {
        var destinationGeocodeOptions = { 'address': addressValue };
      }

      // remove any errors that might have been there from last submit
      $destinationAddressInput.removeClass('quickcab-field-error');
      formObject.$form.find('.qc_form_warning_quickcab_form_destination_address_' + formObject.formId).slideUp();

      // add address to hidden input to be saved when order placed
      $destinationAddressInput.siblings('.quickcab-hidden-autocomplete-field').val(addressValue);
      formObject.destination.address = addressValue;

      // geocode address to get coords
      geocoder.geocode(destinationGeocodeOptions, function(results, status) {
        if (status == google.maps.GeocoderStatus.OK) {
          quickcabHandleDestinationGeocode(formObject, results);
        }
      });
    });

    // when enter pressed do nothing
    google.maps.event.addDomListener($destinationAddressInput.get(0), 'keydown', function(e) {
      if (e.keyCode == 13) {
        e.preventDefault();
      }
    });

    $destinationAddressInput.on('input', function() {
      jQuery(this).siblings('.quickcab-hidden-autocomplete-field').val('');
    });
  }
}

function quickcabHandleOriginGeocode(formObject, results) {
  var latLng = { lat: results[0].geometry.location.lat(), lng: results[0].geometry.location.lng() };

  formObject.origin.lat = latLng.lat;
  formObject.origin.lng = latLng.lng;

  formObject.$form.find('.quickcab-origin-coords-lat').val(latLng.lat);
  formObject.$form.find('.quickcab-origin-coords-lng').val(latLng.lng);

  quickcabPlotOriginMarker(formObject, latLng);
  quickcabCalculateRoute(formObject);
}

function quickcabHandleDestinationGeocode(formObject, results) {
  var latLng = { lat: results[0].geometry.location.lat(), lng: results[0].geometry.location.lng() };

  formObject.destination.lat = latLng.lat;
  formObject.destination.lng = latLng.lng;

  formObject.$form.find('.quickcab-destination-coords-lat').val(latLng.lat);
  formObject.$form.find('.quickcab-destination-coords-long').val(latLng.lng);

  quickcabPlotDestinationMarker(formObject, latLng);
  quickcabCalculateRoute(formObject);
}

function quickcabAddWaypointField($form, index) {
  var waypointContainer = $form.find('.quickcab-waypoint-container');
  var formId = $form.attr('data-form-id');

  var waypointElement = jQuery('<div class="row quickcab-input-row quickcab-waypoint-row" data-index="' + index + '">\
    <label class="control-label">' + quickcab_js_settings.translations.waypoint + ' <span class="quickcab-waypoint-index">' + index + '</span> <span class="quickcab-required">*</span></label>\
    <input type="text" class="form-control quickcab-form-input quickcab-waypoints-active quickcab-form-waypoint" autocomplete="off">\
    <div class="quickcab-waypoint-actions-container -waypoint">\
      <a href="#" class="quickcab-remove-waypoint" data-remove-at="' + index + '">-</a>\
      <a href="#" class="quickcab-add-waypoint" data-insert-at="' + (index + 1) + '">+</a>\
    </div>\
    <input type="hidden" class="quickcab-waypoint-address" name="quickcab_form[waypoints][' + index + '][address]" autocomplete="off">\
    <input type="hidden" class="quickcab-waypoint-lat" name="quickcab_form[waypoints][' + index + '][lat]" autocomplete="off">\
    <input type="hidden" class="quickcab-waypoint-long" name="quickcab_form[waypoints][' + index + '][lng]" autocomplete="off">\
  </div>');

  if ( waypointContainer.children().length == 0 ) {
    waypointContainer.append(waypointElement);
  } else {
    if ( index == 0 ) {
      waypointContainer.children('.quickcab-waypoint-row:eq(' + index + ')').before(waypointElement);
    } else {
      waypointContainer.children('.quickcab-waypoint-row:eq(' + (index - 1) + ')').after(waypointElement);
    }
  }

  var formObject = bookingForms.find(function(obj) {
    return obj.formId == formId;
  });

  var $waypointInput = waypointElement.find('.quickcab-form-waypoint');

  var waypointObject = {
    autocomplete: new google.maps.places.Autocomplete(
      $waypointInput.get(0),
      formObject.autocompleteOptions
    ),
    marker: null,
    lat: null,
    lng: null,
    address: null,
    currentIndex: index
  };

  // when a place has been selected, plot it on the map
  google.maps.event.addListener(waypointObject.autocomplete, 'place_changed', function() {
    var googlePlace = waypointObject.autocomplete.getPlace();
    var addressValue = $waypointInput.val();

    if ( googlePlace.hasOwnProperty('place_id') && googlePlace.place_id.length > 0 ) {
      var geocodeOptions = { 'placeId': googlePlace.place_id };
    } else {
      var geocodeOptions = { 'address': addressValue };
    }

    formObject.$form.find('.quickcab-waypoint-row[data-index="' + waypointObject.currentIndex + '"] .quickcab-waypoint-address').val(addressValue);

    // geocode address to get coords
    geocoder.geocode(geocodeOptions, function(results, status) {
      if (status == google.maps.GeocoderStatus.OK) {
        var latLng = { lat: results[0].geometry.location.lat(), lng: results[0].geometry.location.lng() };

        waypointObject.lat = latLng.lat;
        waypointObject.lng = latLng.lng;
        waypointObject.address = addressValue;

        formObject.$form.find('.quickcab-waypoint-row[data-index="' + waypointObject.currentIndex + '"] .quickcab-waypoint-lat').val(latLng.lat);
        formObject.$form.find('.quickcab-waypoint-row[data-index="' + waypointObject.currentIndex + '"] .quickcab-waypoint-long').val(latLng.lng);

        quickcabPlotWaypointMarker(formObject, waypointObject.currentIndex, latLng);
        quickcabCalculateRoute(formObject);
      }
    });
  });

  // when enter pressed do nothing
  google.maps.event.addDomListener($waypointInput.get(0), 'keydown', function(e) {
    if (e.keyCode == 13) {
      e.preventDefault();
    }
  });

  formObject.waypoints.splice(index, 0, waypointObject);
  quickcabRecalculateWaypointIndices(formObject);
}

function quickcabRecalculateWaypointIndices(formObject) {
  formObject.$form.find('.quickcab-waypoint-row').each(function(index) {
    var $this = jQuery(this);
    $this.attr('data-index', index);
    $this.find('.quickcab-waypoint-index').html(index + 1);
    $this.find('.quickcab-remove-waypoint').attr('data-remove-at', index);
    $this.find('.quickcab-add-waypoint').attr('data-insert-at', index + 1);

    var regex = /(.*)\[[0-9]+\](.*)/g

    // replace 'name' attribute of inputs fields with new indices
    $this.find('.quickcab-waypoint-address').attr('name', $this.find('.quickcab-waypoint-address').attr('name').replace(regex, '$1[' + index + ']$2'));
    $this.find('.quickcab-waypoint-lat').attr('name', $this.find('.quickcab-waypoint-lat').attr('name').replace(regex, '$1[' + index + ']$2'));
    $this.find('.quickcab-waypoint-long').attr('name', $this.find('.quickcab-waypoint-long').attr('name').replace(regex, '$1[' + index + ']$2'));
  });

  // reset referenced index so onclick code knows to look in
  // different location (since it has been moved by newly inserted waypoint)
  for (var i = 0; i < formObject.waypoints.length; i++) {
    formObject.waypoints[i].currentIndex = i;
  }

  quickcabRecalculateMarkerLabels(formObject);
}

function quickcabRemoveWaypointField($form, index) {
  var $waypointContainer = $form.find('.quickcab-waypoint-container');
  var formId = $form.attr('data-form-id');

  $waypointContainer.children('.quickcab-waypoint-row:eq(' + index + ')').remove();

  var formObject = bookingForms.find(function(obj) {
    return obj.formId == formId;
  });

  if ( formObject.waypoints[index].marker ) {
    formObject.waypoints[index].marker.setMap(null);
  }

  formObject.waypoints.splice(index, 1);

  quickcabRecalculateWaypointIndices(formObject);
}

function quickcabPlotOriginMarker(formObject, latLng) {
  if ( formObject.origin.marker ) {
    formObject.origin.marker.setMap(null);
  }

  formObject.origin.marker = new google.maps.Marker({
    position: latLng,
    label: '1',
    map: formObject.map
  });
}

function quickcabPlotDestinationMarker(formObject, latLng) {
  if ( formObject.destination.marker ) {
    formObject.destination.marker.setMap(null);
  }

  formObject.destination.marker = new google.maps.Marker({
    position: latLng,
    label: (formObject.waypoints.length + 2).toString(),
    map: formObject.map
  });
}

function quickcabPlotWaypointMarker(formObject, index, latLng) {
  var waypointObject = formObject.waypoints[index];
  
  if ( waypointObject.marker ) {
    waypointObject.marker.setMap(null);
  }

  waypointObject.marker = new google.maps.Marker({
    position: latLng,
    label: (index + 2).toString(),
    map: formObject.map
  });

  quickcabRecalculateMarkerLabels(formObject);
}

function quickcabRecalculateMarkerLabels(formObject) {
  // recalculate label of subsequent markers
  for ( var i = 0; i < formObject.waypoints.length; i++ ) {
    if ( formObject.waypoints[i].marker ) {
      formObject.waypoints[i].marker.setLabel((i + 2).toString());
    }
  }

  if ( formObject.destination.marker ) {
    formObject.destination.marker.setLabel((i + 2).toString());
  }
}

function quickcabCalculateRoute(formObject) {
  // do not calculate if user hasn't entered origin and destination
  if (
    !formObject.origin.lat ||
    !formObject.origin.lng ||
    !formObject.destination.lat ||
    !formObject.destination.lng
  ) {
    return;
  }

  var waypoints = formObject.waypoints.filter(function(waypoint) {
    return waypoint.lat && waypoint.lng && waypoint.address;
  }).map(function(waypoint) {
    return {
      location: new google.maps.LatLng(waypoint.lat, waypoint.lng),
      stopover: true
    }
  });

  var directionsRequest = {
    origin: {
      lat: formObject.origin.lat,
      lng: formObject.origin.lng
    },
    destination: {
      lat: formObject.destination.lat,
      lng: formObject.destination.lng
    },
    waypoints: waypoints,
    travelMode: google.maps.TravelMode.DRIVING,
    unitSystem: quickcab_js_settings.uses_miles ? google.maps.UnitSystem.IMPERIAL : google.maps.UnitSystem.METRIC
  };

  // pass the directions request to the directions service
  directionsService.route(directionsRequest, function(response, status) {
    if (status == google.maps.DirectionsStatus.OK) {
      formObject.directionsDisplay.setMap(formObject.map);
      formObject.directionsDisplay.setDirections(response); // display the route on the map

      google.maps.event.trigger(formObject.map, 'resize');
      formObject.map.setCenter(response.routes[0].bounds.getCenter());
      formObject.map.fitBounds(response.routes[0].bounds);

      // distance in metres
      var totalDistance = response.routes[0].legs.map(function(leg) {
        return leg.distance.value;
      }).reduce(function(total, num) {
        return total + num;
      });

      // duration in seconds
      var totalDuration = response.routes[0].legs.map(function(leg) {
        return leg.duration.value;
      }).reduce(function(total, num) {
        return total + num;
      });

      formObject.routeStats.totalDistance = totalDistance;
      formObject.routeStats.totalDuration = totalDuration;

      var distanceDenominator = quickcab_js_settings.uses_miles ? 1609.34 : 1000;
      var unit = quickcab_js_settings.uses_miles ? quickcab_js_settings.translations.miles : quickcab_js_settings.translations.kilometers;

      var distance = quickcabRoundToOne(totalDistance / distanceDenominator) + ' ' + unit;
      var duration = quickcabMinsToText(totalDuration / 60);

      if ( formObject.showDistance ) {
        formObject.$form.find('.quickcab-map-stat-text.-distance').html(distance);
        formObject.$form.find('.quickcab-trip-preview-text.-distance').html(distance);
      }

      if ( formObject.showDuration ) {
        formObject.$form.find('.quickcab-map-stat-text.-duration').html(duration);
        formObject.$form.find('.quickcab-trip-preview-text.-duration').html(duration);
      }
    }
  });
}

function quickcabGetVehiclePrices(formObject) {
  var waypoints = formObject.waypoints.filter(function(waypoint) {
    return waypoint.lat && waypoint.lng && waypoint.address;
  }).map(function(waypoint) {
    return {
      lat: waypoint.lat,
      lng: waypoint.lng,
      address: waypoint.address
    }
  });

  var data = {
    action: 'quickcab_get_quotes',
    quickcab_form: {
      id: parseInt(formObject.$form.find('.quickcab-form-id').val()),
      occupants: parseInt(formObject.$form.find('.quickcab-form-occupants :selected').val()),
      suitcases: parseInt(formObject.$form.find('.quickcab-form-suitcases :selected').val()),
      direction: formObject.$form.find('.quickcab-form-direction :selected').val(),
      departure_date: formObject.$form.find('.quickcab-form-departure-date').val(),
      departure_time: formObject.$form.find('.quickcab-form-departure-time').val(),
      return_departure_date: formObject.$form.find('.quickcab-form-return-departure-date').val(),
      return_departure_time: formObject.$form.find('.quickcab-form-return-departure-time').val(),
      extra_waiting_time: parseInt(formObject.$form.find('.quickcab-form-waiting-time').val()),
      origin: {
        lat: formObject.origin.lat,
        lng: formObject.origin.lng,
        address: formObject.origin.address
      },
      destination: {
        lat: formObject.destination.lat,
        lng: formObject.destination.lng,
        address: formObject.destination.address
      },
      waypoints: waypoints
    }
  };

  jQuery.ajax({
    data: data,
    type: 'post',
    url: quickcab_js_settings.ajax_url,
    success: function(return_data) {
      var parsed_data = JSON.parse(return_data);
      pricing_data = parsed_data.prices;

      var $findVehiclesButton = formObject.$form.find('.quickcab-booking-form1-section1-submit');
      $findVehiclesButton.text($findVehiclesButton.attr('data-original-text'));

      formObject.$form.find('.quickcab-vehicles-list').empty();

      if ( parsed_data.can_book ) {
        for ( var i = 0; i < parsed_data['prices'].length; i++ ) {
          var vehiclePassengers = parsed_data['prices'][i]['vehicle']['max_occupants'];
          var vehicleBags = parsed_data['prices'][i]['vehicle']['max_suitcases'];
          var totalPrice = parsed_data['prices'][i]['quote']['total_breakdown']['total']['amount_formatted'];
          var vehicleID = parsed_data['prices'][i]['vehicle']['id'];

          var vehicleFeatures = '<ul class="quickcab-vehicle-features">';

          for (var j = 0; j < parsed_data['prices'][i]['vehicle']['features'].length; j++) {
            var featureText = parsed_data['prices'][i]['vehicle']['features'][j]['name'];
            vehicleFeatures += '<li><span class="glyphicon glyphicon-ok" aria-hidden="true"></span> ' + featureText + '</li>';
          }

          vehicleFeatures += '</ul>';

          var passengerInfo = wpdk_apply_filters('quickcab_vehicle_result_passenger_info', '<div class="quickcab-vehicle-icon">\
            <span class="glyphicon glyphicon-user" aria-hidden="true"></span> <span class="quickcab-vehicle-icon-number">' + vehiclePassengers + '</span>\
          </div>', vehicleID);

          var suitcaseInfo = wpdk_apply_filters('quickcab_vehicle_result_suitcase_info', '<div class="quickcab-vehicle-icon">\
            <span class="glyphicon glyphicon-briefcase" aria-hidden="true"></span> <span class="quickcab-vehicle-icon-number">' + vehicleBags + '</span>\
          </div>', vehicleID);

          var htmlResult = '\
            <div class="quickcab-vehicle-item">\
              <div class="row">\
                <div class="col-md-4 quickcab-vehicle-image-column">\
                  <img src="' + parsed_data['prices'][i]['vehicle']['image_url'] + '" alt="">\
                </div>\
                <div class="col-md-8 quickcab-vehicle-details-column">\
                  <div class="pull-right">\
                    <div class="quickcab-book-container">\
                      <div class="quickcab-vehicle-price">' + totalPrice + '</div>\
                      <a href="#" class="quickcab-button -inverted quickcab-vehicle-book-button" data-vehicle-id="' + vehicleID + '" data-index="' + i + '" data-disable-with="' + quickcab_js_settings.form.book_vehicle_button_disable_text + '" data-original-text="' + quickcab_js_settings.form.book_vehicle_button_text + '">' + quickcab_js_settings.form.book_vehicle_button_text + '</a>\
                    </div>\
                    <div class="quickcab-vehicle-icons">' + passengerInfo + suitcaseInfo + '</div>\
                  </div>\
                  <div class="quickcab-vehicle-name">' + parsed_data['prices'][i]['vehicle']['name'] + '</div>\
                  ' + vehicleFeatures + '\
                </div>\
              </div>\
            </div>\
          ';

          formObject.$form.find('.quickcab-vehicles-list').append(htmlResult);
        }

        quickcabMoveToTab(formObject, 1);
      } else {
        formObject.$form.find('.quickcab-error-message').html(parsed_data.message);
        formObject.$form.find('.quickcab-error').show();
      }

      jQuery([document.documentElement, document.body]).animate({
        scrollTop: formObject.$form.offset().top
      }, 400);
    },
    error: function() {
      var $findVehiclesButton = formObject.$form.find('.quickcab-booking-form1-section1-submit');
      $findVehiclesButton.text($findVehiclesButton.attr('data-original-text'));

      formObject.$form.find('.quickcab-error-message').html(quickcab_js_settings.translations.error_submitting_form);
      formObject.$form.find('.quickcab-error').show();

      jQuery([document.documentElement, document.body]).animate({
        scrollTop: formObject.$form.offset().top
      }, 400);
    }
  });
}

function quickcabMoveToTab(formObject, index) {
  if ( index >= formObject.activeTabs.length ) {
    return;
  }

  formObject.activeTab = index;

  for ( var i = 0; i < formObject.activeTabs.length; i++ ) {
    var is_active = i <= index;
    formObject.activeTabs[i] = is_active;

    if ( is_active ) {
      formObject.$form.find('.quickcab-cart-steps li.-section' + i).addClass('active');
    } else {
      formObject.$form.find('.quickcab-cart-steps li.-section' + i).removeClass('active');
    }

    if ( i == index ) {
      formObject.$form.find('.form-section' + i).show();
    } else {
      formObject.$form.find('.form-section' + i).hide();
    }
  }
}

function quickcabErrorInCurrentSection(formObject) {
  var section = formObject.requiredFields[formObject.activeTab];

  // if return journey add return times to required
  if ( formObject.$form.find('.quickcab-form-direction :selected').val() === 'return' ) {
    section['text'].push('quickcab_form_return_departure_date_' + formObject.formId);
    section['select'].push('quickcab_form_return_departure_time_' + formObject.formId);
  } else {
    var retDateIndex = section['text'].indexOf('quickcab_form_return_departure_date_' + formObject.formId);
    var retTimeIndex = section['select'].indexOf('quickcab_form_return_departure_time_' + formObject.formId);

    if ( retDateIndex > -1 ) {
      section['text'].splice(retDateIndex, 1);
    }

    if ( retTimeIndex > -1 ) {
      section['select'].splice(retTimeIndex, 1);
    }
  }

  for ( var i = 0; i < section['text'].length; i++ ) {
    var textError = jQuery('#' + section['text'][i]).val().length == 0;
    var filteredTextError = wpdk_apply_filters('quickcab_form_text_error', textError, section['text'][i], formObject);

    if ( filteredTextError ) {
      return filteredTextError;
    }
  }

  for ( var i = 0; i < section['google_autocomplete'].length; i++ ) {
    var autocompleteError = section['google_autocomplete'][i]['var'].getPlace() === undefined ||
      jQuery('#' + section['google_autocomplete'][i]['id']).siblings('.quickcab-hidden-autocomplete-field').val().length === 0;
    var filteredAutocompleteError = wpdk_apply_filters('quickcab_form_autocomplete_error', autocompleteError, section['google_autocomplete'][i], formObject);

    if ( filteredAutocompleteError ) {
      return filteredAutocompleteError;
    }
  }

  for ( var i = 0; i < section['select'].length; i++ ) {
    var selectError = jQuery('#' + section['select'][i]).val() == null;
    var filteredSelectError = wpdk_apply_filters('quickcab_form_select_error', selectError, section['select'][i], formObject);

    if ( filteredSelectError ) {
      return filteredSelectError;
    }
  }

  return false;
}

function quickcabHighlightErrorsInForm(formObject) {
  var error = false;
  var section = formObject.requiredFields[formObject.activeTab];

  // if return journey add return times to required
  if ( formObject.$form.find('.quickcab-form-direction :selected').val() === 'return' ) {
    section['text'].push('quickcab_form_return_departure_date_' + formObject.formId);
    section['select'].push('quickcab_form_return_departure_time_' + formObject.formId);
  } else {
    var retDateIndex = section['text'].indexOf('quickcab_form_return_departure_date_' + formObject.formId);
    var retTimeIndex = section['select'].indexOf('quickcab_form_return_departure_time_' + formObject.formId);

    if ( retDateIndex > -1 ) {
      section['text'].splice(retDateIndex, 1);
    }

    if ( retTimeIndex > -1 ) {
      section['select'].splice(retTimeIndex, 1);
    }
  }

  for ( var i = 0; i < section['text'].length; i++ ) {
    jQuery('#' + section['text'][i]).removeClass('quickcab-field-error');
    var textError = jQuery('#' + section['text'][i]).val().length == 0;
    var filteredTextError = wpdk_apply_filters('quickcab_form_text_error', textError, section['text'][i], formObject);

    if ( filteredTextError ) {
      jQuery('#' + section['text'][i]).addClass('quickcab-field-error');
    }
  }

  for ( var i = 0; i < section['google_autocomplete'].length; i++ ) {
    jQuery('#' + section['google_autocomplete'][i]['id']).removeClass('quickcab-field-error');
    var autocompleteError = section['google_autocomplete'][i]['var'].getPlace() === undefined ||
      jQuery('#' + section['google_autocomplete'][i]['id']).siblings('.quickcab-hidden-autocomplete-field').val().length === 0;
    var filteredAutocompleteError = wpdk_apply_filters('quickcab_form_autocomplete_error', autocompleteError, section['google_autocomplete'][i], formObject);

    if ( filteredAutocompleteError ) {
      jQuery('#' + section['google_autocomplete'][i]['id']).addClass('quickcab-field-error');
      jQuery('.qc_form_warning_' + section['google_autocomplete'][i]['id']).slideDown();
    } else {
      jQuery('.qc_form_warning_' + section['google_autocomplete'][i]['id']).slideUp();
    }
  }

  for ( var i = 0; i < section['select'].length; i++ ) {
    jQuery('#' + section['select'][i]).removeClass('quickcab-field-error');
    var selectError = jQuery('#' + section['select'][i]).val() == null;
    var filteredSelectError = wpdk_apply_filters('quickcab_form_select_error', selectError, section['select'][i], formObject);

    if ( filteredSelectError ) {
      jQuery('#' + section['select'][i]).addClass('quickcab-field-error');
    }
  }
}

jQuery('.quickcab-booking-form1-section1-submit').on('click', function(e) {
  e.preventDefault();

  var $form = jQuery(this).closest('.quickcab-booking-form');
  var formId = parseInt($form.attr('data-form-id'));
  
  var formObject = bookingForms.find(function(formObject) {
    return formObject.formId == formId;
  });

  var startingAddress = formObject.$form.find('.quickcab-form-starting-address').val();
  var destinationAddress = formObject.$form.find('.quickcab-form-destination-address').val();

  if ( quickcab_js_settings.forms[formObject.formId].use_fixed_addresses_starting ) {
    startingAddress = formObject.$form.find('.quickcab-form-starting-address option:selected').text();
  }

  if ( quickcab_js_settings.forms[formObject.formId].use_fixed_addresses_destination ) {
    destinationAddress = formObject.$form.find('.quickcab-form-destination-address.quickcab-select-input option:selected').text();
  }
  var collectionTime = formObject.$form.find('.quickcab-form-departure-date').val() + " &mdash; " + formObject.$form.find('.quickcab-form-departure-time :selected').val();
  var returnCollectionTime = formObject.$form.find('.quickcab-form-return-departure-date').val() + " &mdash; " + formObject.$form.find('.quickcab-form-return-departure-time :selected').val();

  formObject.$form.find('.quickcab-error').hide();

  if ( quickcabErrorInCurrentSection(formObject) ) {
    quickcabHighlightErrorsInForm(formObject);
    formObject.$form.find('.quickcab-field-error').first().focus();
  } else {
    var $this = jQuery(this);

    $this.prop('disabled', true);
    $this.text($this.attr('data-disable-with'));

    formObject.$form.find('.quickcab-trip-preview-text.-route').html(startingAddress + " &mdash; " + destinationAddress);
    formObject.$form.find('.quickcab-trip-preview-text.-collection-time').html(collectionTime);

    if ( formObject.$form.find('.quickcab-form-direction :selected').val() == 'return' ) {
      formObject.$form.find('.quickcab-trip-preview-title.-return-collection-time').show();
      formObject.$form.find('.quickcab-trip-preview-text.-return-collection-time').show();
      formObject.$form.find('.quickcab-trip-preview-text.-return-collection-time').html(returnCollectionTime);
    } else {
      formObject.$form.find('.quickcab-trip-preview-title.-return-collection-time').hide();
      formObject.$form.find('.quickcab-trip-preview-text.-return-collection-time').hide();
    }

    quickcabGetVehiclePrices(formObject);
  }
});

jQuery('.quickcab-booking-form1 .cart-step-link').on('click', function(e) {
  e.preventDefault();
  var index = parseInt(jQuery(this).attr('data-index'));

  var $form = jQuery(this).closest('.quickcab-booking-form');
  var formId = parseInt($form.attr('data-form-id'));
  var formObject = bookingForms.find(function(formObject) {
    return formObject.formId == formId;
  });

  if ( formObject.activeTabs[index] ) {
    quickcabMoveToTab(formObject, index);
  }
});

function quickcabRoundToOne(num) {
  return +(Math.round(num + "e+1")  + "e-1");
}

function quickcabMinsToText(n) {
  var num = n;
  var hours = (num / 60);
  var rhours = Math.floor(hours);
  var minutes = (hours - rhours) * 60;
  var rminutes = Math.round(minutes);
  var returnString = "";

  if ( rhours > 0 ) {
    var hourQualifier = rhours == 1 ? quickcab_js_settings.translations.hour : quickcab_js_settings.translations.hours;
    returnString += rhours + ' ' + hourQualifier + ' ';
  }

  var minQualifier = rhours == 1 ? quickcab_js_settings.translations.minute : quickcab_js_settings.translations.minutes;
  returnString += rminutes + ' ' + minQualifier;

  return returnString;
}
