<?php

if ( !defined('ABSPATH') ) exit;

function quickcab_add_booking_form_meta_box() {
  add_meta_box('quickcab_booking_form_options', esc_html__('Form Options', 'quickcab'),  'quickcab_render_booking_form_options_meta_box', array('qc_booking_form'), 'normal', 'default');
  add_meta_box('quickcab_booking_form_field_options', esc_html__('Fields', 'quickcab'),  'quickcab_render_booking_form_field_options_meta_box', array('qc_booking_form'), 'normal', 'default');
  add_meta_box('quickcab_booking_form_map_options', esc_html__('Map Options', 'quickcab'),  'quickcab_render_booking_form_map_options_meta_box', array('qc_booking_form'), 'normal', 'default');
}
add_action('add_meta_boxes', 'quickcab_add_booking_form_meta_box');

function quickcab_render_booking_form_options_meta_box() {
  global $post;

  do_action('quickcab_meta_box_booking_form_options_fields', $post->ID);

  wp_nonce_field(basename(__FILE__), 'quickcab_booking_form_meta_box_nonce');
}

function quickcab_render_booking_form_field_options_meta_box() {
  global $post;

  do_action('quickcab_meta_box_booking_form_field_options_fields', $post->ID);
}

function quickcab_render_booking_form_map_options_meta_box() {
  global $post;

  do_action('quickcab_meta_box_booking_form_map_option_fields', $post->ID);
}

function quickcab_render_booking_form_options_fields($post_id) {
  $screen = get_current_screen();
  $is_new = $screen->action == 'add';

  if ( !$is_new ) {
    $booking_form = new QuickCab_Booking_Form($post_id);
  }

  ?><table class="form-table">
    <tbody><?php
      if ( !$is_new ) {
        quickcab_render_metabox_code_row(
          'quickcab_booking_form_shortcode',
          $post_id,
          array(
            'text' => esc_html__('Shortcode', 'quickcab')
          ),
          array(
            'value' => '[quickcab_form id="' . $booking_form->get_ID() .'"]'
          )
        );
      }
      quickcab_render_booking_form_metabox_colorpicker_row(
        'quickcab_booking_form_primary_color',
        $post_id,
        array(
          'text' => esc_html__('Primary Color', 'quickcab')
        ),
        array(
          'required' => true,
          'default_value' => '#1080f2'
        ),
        array(
          'text' => esc_html__('The primary color of the form (button color etc.)', 'quickcab')
        )
      );
      quickcab_render_booking_form_metabox_colorpicker_row(
        'quickcab_booking_form_secondary_color',
        $post_id,
        array(
          'text' => esc_html__('Secondary Color', 'quickcab')
        ),
        array(
          'required' => true,
          'default_value' => '#fff'
        ),
        array(
          'text' => esc_html__('The secondary color of the form (button text etc.)', 'quickcab')
        )
      );
      quickcab_render_booking_form_metabox_select_row(
        'quickcab_booking_form_booking_time_interval',
        $post_id,
        array(
          'text' => esc_html__('Booking Time Interval (minutes)', 'quickcab')
        ),
        array(
          'options' => array(
            '5' => '5',
            '10' => '10',
            '15' => '15',
            '30' => '30',
            '60' => '60'
          )
        ),
        array(
          'text' => esc_html__('The number of minutes between each booking time option in the form.', 'quickcab')
        )
      );
      quickcab_render_booking_form_metabox_checkbox_row(
        'quickcab_booking_form_fixed_address_collection',
        $post_id,
        array(
          'text' => esc_html__('Used Fixed Addresses for Collection Address', 'quickcab')
        ),
        array(
          'size' => 'medium'
        ),
        array(
          'text' => esc_html__('Do you want to use Fixed Addresses for the Collection Address field? You can create Fixed Addresses in Sidebar > QuickCab > Fixed Addresses.', 'quickcab')
        )
      );
      quickcab_render_booking_form_metabox_checkbox_row(
        'quickcab_booking_form_fixed_address_destination',
        $post_id,
        array(
          'text' => esc_html__('Used Fixed Addresses for Destination Address', 'quickcab')
        ),
        array(
          'size' => 'medium'
        ),
        array(
          'text' => esc_html__('Do you want to use Fixed Addresses for the Destination Address field? You can create Fixed Addresses in Sidebar > QuickCab > Fixed Addresses.', 'quickcab')
        )
      );
      quickcab_render_booking_form_metabox_checkbox_row(
        'quickcab_booking_form_enforce_autocomplete_restriction',
        $post_id,
        array(
          'text' => esc_html__('Enforce Autocomplete Country Restriction?', 'quickcab')
        ),
        array(
          'size' => 'medium'
        ),
        array(
          'text' => esc_html__('Do you want to restrict the autocomplete values in the form to locations in a specific country?', 'quickcab')
        )
      );
      quickcab_render_booking_form_metabox_select_row(
        'quickcab_booking_form_autocomplete_country',
        $post_id,
        array(
          'text' => esc_html__('Autocomplete Country', 'quickcab')
        ),
        array(
          'options' => quickcab_iso_3166_country_codes(),
          'disabled' => $is_new || !$booking_form->get_enforce_autocomplete_restriction()
        ),
        array(
          'text' => esc_html__('Only options from this country will display in the Google Maps autocomplete in the form.', 'quickcab')
        )
      );
      quickcab_render_booking_form_metabox_checkbox_row(
        'quickcab_booking_form_enable_waypoints',
        $post_id,
        array(
          'text' => esc_html__('Enable Waypoints', 'quickcab')
        ),
        array(
          'size' => 'medium'
        ),
        array(
          'text' => esc_html__('Allow the customer to add additional waypoints to the journey, ', 'quickcab')
        )
      );
    ?></tbody>
  </table><?php
}
add_action('quickcab_meta_box_booking_form_options_fields', 'quickcab_render_booking_form_options_fields');

function quickcab_render_booking_form_field_options_fields($post_id) {
  $screen = get_current_screen();
  $is_new = $screen->action == 'add';

  if ( !$is_new ) {
    $booking_form = new QuickCab_Booking_Form($post_id);
  }

  ?><table class="form-table">
    <tbody><?php
      quickcab_render_booking_form_metabox_checkbox_row(
        'quickcab_booking_form_enable_extra_waiting_time',
        $post_id,
        array(
          'text' => esc_html__('Enable Extra Waiting Time', 'quickcab')
        ),
        array(
          'size' => 'medium'
        ),
        array(
          'text' => esc_html__('Allow the customer to add extra waiting time to the booking. The price for this can be set in the vehicle settings.', 'quickcab')
        )
      );
      quickcab_render_booking_form_metabox_select_row(
        'quickcab_booking_form_extra_waiting_time_unit',
        $post_id,
        array(
          'text' => esc_html__('Extra Waiting Time Unit', 'quickcab')
        ),
        array(
          'options' => array(
            'hours' => esc_html__('Hours', 'quickcab'),
            'minutes' => esc_html__('Minutes', 'quickcab')
          ),
          'disabled' => $is_new || !$booking_form->enable_extra_waiting_time()
        ),
        array(
          'text' => esc_html__('The units used for the extra waiting time field.', 'quickcab')
        )
      );
      quickcab_render_booking_form_metabox_number_row(
        'quickcab_booking_form_extra_waiting_time_max_value',
        $post_id,
        array(
          'text' => esc_html__('Extra Waiting Time Max Value', 'quickcab')
        ),
        array(
          'disabled' => $is_new || !$booking_form->enable_extra_waiting_time(),
          'step' => 1,
          'value' => $is_new ? '10' : $booking_form->get_extra_waiting_time_max_value(),
          'suffix' => ' <span class="quickcab-booking-form-extra-time-unit">' . ($is_new ? esc_html__('hours', 'quickcab') : $booking_form->get_extra_waiting_time_unit_i18n()) . '</span>'
        ),
        array(
          'text' => esc_html__('The maximum value a user can select for extra waiting time in the form.', 'quickcab')
        )
      );
      quickcab_render_booking_form_metabox_checkbox_row(
        'quickcab_booking_form_show_distance',
        $post_id,
        array(
          'text' => esc_html__('Show Distance', 'quickcab')
        ),
        array(
          'checked' => $is_new || $booking_form->show_distance()
        ),
        array(
          'text' => esc_html__('Show the user the calculated distance of the journey.', 'quickcab')
        )
      );
      quickcab_render_booking_form_metabox_checkbox_row(
        'quickcab_booking_form_show_duration',
        $post_id,
        array(
          'text' => esc_html__('Show Duration', 'quickcab')
        ),
        array(
          'checked' => $is_new || $booking_form->show_duration()
        ),
        array(
          'text' => esc_html__('Show the user the calculated duration of the journey.', 'quickcab')
        )
      );
      quickcab_render_booking_form_metabox_checkbox_row(
        'quickcab_booking_form_enable_passengers_field',
        $post_id,
        array(
          'text' => esc_html__('Enable Passengers Field', 'quickcab')
        ),
        array(
          'checked' => $is_new || $booking_form->enable_passengers_field()
        ),
        array(
          'text' => esc_html__('Allow the user to select number of passengers.', 'quickcab')
        )
      );
      quickcab_render_booking_form_metabox_checkbox_row(
        'quickcab_booking_form_enable_suitcases_field',
        $post_id,
        array(
          'text' => esc_html__('Enable Suitcases Field', 'quickcab')
        ),
        array(
          'checked' => $is_new || $booking_form->enable_suitcases_field()
        ),
        array(
          'text' => esc_html__('Allow the user to select number of suitcases.', 'quickcab')
        )
      );
      quickcab_render_booking_form_metabox_checkbox_row(
        'quickcab_booking_form_enable_direction_field',
        $post_id,
        array(
          'text' => esc_html__('Enable Direction Field', 'quickcab')
        ),
        array(
          'checked' => $is_new || $booking_form->enable_direction_field()
        ),
        array(
          'text' => esc_html__('Allow the user to select between one way and return journey.', 'quickcab')
        )
      );
    ?></tbody>
  </table><?php
}
add_action('quickcab_meta_box_booking_form_field_options_fields', 'quickcab_render_booking_form_field_options_fields');

function quickcab_render_booking_form_map_option_fields($post_id) {
  $screen = get_current_screen();
  $is_new = $screen->action == 'add';

  if ( !$is_new ) {
    $booking_form = new QuickCab_Booking_Form($post_id);
  }

  ?><table class="form-table">
    <tbody><?php
      quickcab_render_booking_form_metabox_autocomplete_row(
        'quickcab_booking_form_map_starting_location',
        $post_id,
        array(
          'text' => esc_html__('Map Starting Location', 'quickcab')
        ),
        array(),
        array(
          'text' => esc_html__('The location the map is initialy centered on when the page loads.', 'quickcab')
        )
      );
      quickcab_render_booking_form_metabox_checkbox_row(
        'quickcab_booking_form_enable_zoom_control',
        $post_id,
        array(
          'text' => esc_html__('Enable Zoom Control', 'quickcab')
        ),
        array(
          'checked' => $is_new || $booking_form->enable_zoom_control()
        ),
        array(
          'text' => esc_html__('Allow the user to zoom in and out on the booking form map.', 'quickcab')
        )
      );
      quickcab_render_booking_form_metabox_checkbox_row(
        'quickcab_booking_form_enable_map_type_control',
        $post_id,
        array(
          'text' => esc_html__('Enable Map Type Control', 'quickcab')
        ),
        array(
          'checked' => $is_new || $booking_form->enable_map_type_control()
        ),
        array(
          'text' => esc_html__('Allow the user to be able to select the map type (e.g. Satellite, Terrain).', 'quickcab')
        )
      );
      quickcab_render_booking_form_metabox_checkbox_row(
        'quickcab_booking_form_enable_street_view_control',
        $post_id,
        array(
          'text' => esc_html__('Enable Street View Control', 'quickcab')
        ),
        array(
          'checked' => $is_new || $booking_form->enable_street_view_control()
        ),
        array(
          'text' => esc_html__('Allow the user to go into street view.', 'quickcab')
        )
      );
      quickcab_render_booking_form_metabox_checkbox_row(
        'quickcab_booking_form_enable_full_screen_control',
        $post_id,
        array(
          'text' => esc_html__('Enable Full Screen Control', 'quickcab')
        ),
        array(
          'checked' => $is_new || $booking_form->enable_full_screen_control()
        ),
        array(
          'text' => esc_html__('Allow the user to go into full screen mode.', 'quickcab')
        )
      );
      quickcab_render_booking_form_metabox_checkbox_row(
        'quickcab_booking_form_enable_pan_control',
        $post_id,
        array(
          'text' => esc_html__('Enable Pan Control', 'quickcab')
        ),
        array(
          'checked' => $is_new || $booking_form->enable_pan_control()
        ),
        array(
          'text' => esc_html__('Allow the user to pan the map (i.e. drag to move around).', 'quickcab')
        )
      );
    ?></tbody>
  </table><?php
}
add_action('quickcab_meta_box_booking_form_map_option_fields', 'quickcab_render_booking_form_map_option_fields');

function quickcab_booking_form_metabox_fields() {
  return apply_filters('quickcab_metabox_booking_form_fields_save', array(
    'quickcab_booking_form_primary_color',
    'quickcab_booking_form_secondary_color',
    'quickcab_booking_form_map_starting_location',
    'quickcab_booking_form_booking_time_interval',
    'quickcab_booking_form_fixed_address_collection',
    'quickcab_booking_form_fixed_address_destination',
    'quickcab_booking_form_enforce_autocomplete_restriction',
    'quickcab_booking_form_autocomplete_country',
    'quickcab_booking_form_enable_extra_waiting_time',
    'quickcab_booking_form_extra_waiting_time_unit',
    'quickcab_booking_form_extra_waiting_time_max_value',
    'quickcab_booking_form_enable_waypoints',
    'quickcab_booking_form_enable_zoom_control',
    'quickcab_booking_form_enable_map_type_control',
    'quickcab_booking_form_enable_street_view_control',
    'quickcab_booking_form_enable_full_screen_control',
    'quickcab_booking_form_enable_pan_control',
    'quickcab_booking_form_show_distance',
    'quickcab_booking_form_show_duration',
    'quickcab_booking_form_enable_direction_field',
    'quickcab_booking_form_enable_passengers_field',
    'quickcab_booking_form_enable_suitcases_field'
  ));
}

/**
 * Run after the post is saved, saves all data in metaboxes
 *
 * @since  1.0
 * @param  int     $post_id  ID of the post
 * @param  WP_POST $post     the post being saved
 * @return void
 */
function quickcab_booking_form_meta_box_save($post_id, $post) {
  if ( !isset($_POST['quickcab_booking_form_meta_box_nonce']) || !wp_verify_nonce(wp_unslash($_POST['quickcab_booking_form_meta_box_nonce']), basename(__FILE__)) ) {
    return;
  }

  if ( (defined('DOING_AUTOSAVE') && DOING_AUTOSAVE) || (defined('DOING_AJAX') && DOING_AJAX) || isset($_REQUEST['bulk_edit']) ) {
    return;
  }

  if ( isset($post->post_type) && 'revision' == $post->post_type ) {
    return;
  }

  $booking_form = new QuickCab_Booking_Form($post_id);

  foreach ( quickcab_booking_form_metabox_fields() as $field ) {
    $meta_name = str_replace('quickcab_booking_form_', '', $field);

    if ( empty($_POST[$field]) ) {
      $booking_form->delete_meta($meta_name);
    } else {
      $new = apply_filters('quickcab_booking_form_metabox_save_' . $meta_name, wp_unslash($_POST[$field]), $post_id);
      $booking_form->update_meta($meta_name, $new);
    }
  }

  do_action('quickcab_save_booking_form', $post_id, $post);
}
add_action('save_post', 'quickcab_booking_form_meta_box_save', 10, 2);
