<?php

if ( !defined('ABSPATH') ) exit;

function quickcab_add_order_meta_box() {
  global $post;

  $screen = get_current_screen();
  $is_new = $screen->action == 'add';
  $quickcab_order = new QuickCab_Order($post->ID);

  if ( !empty($quickcab_order->get_starting_address()) ) {
    add_meta_box('quickcab_order_details', esc_html__('Order Details', 'quickcab'),  'quickcab_render_order_details_meta_box', array('shop_order'), 'normal', 'high');
  }

  remove_meta_box('submitdiv', 'quickcab_order', 'side');
}
add_action('add_meta_boxes', 'quickcab_add_order_meta_box');

function quickcab_render_order_details_meta_box() {
  global $post;

  do_action('quickcab_meta_box_order_details_fields', $post->ID);

  wp_nonce_field(basename(__FILE__), 'quickcab_order_meta_box_nonce');
}

function quickcab_render_order_details_fields($post_id) {
  $order = new QuickCab_Order($post_id);
  $booking_form = $order->get_booking_form();

  ?><div class="quickcab-order-details-row">
    <div class="quickcab-order-details-column">
      <div id="quickcab_order_map"></div>
    </div>
    <div class="quickcab-order-details-column -details">
      <div class="quickcab-order-details-row">
        <div class="quickcab-destinations-container">
          <div class="quickcab-destinations">
            <div id="quickcab_timeline">
              <div>
                <section class="year">
                  <section>
                    <ul>
                      <li>
                        <a href="http://maps.google.com/?q=<?php echo urlencode(esc_html($order->get_starting_address())); ?>" target="_blank" class="quickcab-order-details__address"><?php
                          echo esc_html($order->get_starting_address());
                        ?></a>
                      </li>
                    </ul>
                  </section><?php
                  $waypoints = $order->get_waypoints();
                  if ( count($waypoints) > 0 ) {
                    for ( $i = 0; $i < count($waypoints); $i++ ) {
                      ?><section>
                        <ul class="-quickcab-waypoint">
                          <li>
                            <a href="http://maps.google.com/?q=<?php echo urlencode(esc_html($waypoints[$i]['address'])); ?>" target="_blank" class="quickcab-order-details__address"><?php
                              echo esc_html($waypoints[$i]['address']);
                            ?></a>
                          </li>
                        </ul>
                      </section><?php
                    }
                  }
                  ?><section>
                    <ul>
                      <li>
                        <a href="http://maps.google.com/?q=<?php echo urlencode(esc_html($order->get_destination_address())); ?>" target="_blank" class="quickcab-order-details__address"><?php
                          echo esc_html($order->get_destination_address());
                        ?></a>
                      </li>
                    </ul>
                  </section>
                </section>
              </div>
            </div>
          </div>
        </div>

        <div class="quickcab-order-detail">
          <p>
            <strong class="quickcab-order-detail-label"><?php
              echo esc_html__('Vehicle', 'quickcab');
            ?></strong>
            <span><?php
              if ( get_post_status($order->get_vehicle_id()) ) {
                ?><a href="<?php echo esc_attr(get_edit_post_link($order->get_vehicle_id())); ?>" target="_blank"><?php
                  echo esc_html($order->get_vehicle()->get_name());
                ?></a><?php
              } else {
                echo esc_html__('vehicle deleted', 'quickcab');
              }
            ?></span>
          </p>
        </div>

        <div class="quickcab-order-detail">
          <p>
            <strong class="quickcab-order-detail-label"><?php
              echo esc_html__('One Way', 'quickcab') . '/' . esc_html__('Return', 'quickcab');
            ?></strong>
            <span><?php
              if ( $order->is_return() ) {
                echo esc_html__('Return', 'quickcab');
              } else {
                echo esc_html__('One Way', 'quickcab');
              }
            ?></span>
          </p>
        </div>

        <div class="quickcab-order-detail">
          <p>
            <strong class="quickcab-order-detail-label"><?php
              echo esc_html__('Distance', 'quickcab');
            ?></strong>
            <span><?php
              echo esc_html($order->get_distance_in_words());
            ?></span>
          </p>
        </div>

        <div class="quickcab-order-detail">
          <p>
            <strong class="quickcab-order-detail-label"><?php
              echo esc_html__('Journey Duration', 'quickcab') . ' (' . (empty($order->get_duration_in_words_traffic()) ? esc_html__('no traffic data available', 'quickcab') : esc_html__('including traffic', 'quickcab')) . ')';
            ?></strong>
            <span><?php
              if ( empty($order->get_duration_in_words_traffic()) ) {
                echo esc_html($order->get_duration_in_words());
              } else {
                echo esc_html($order->get_duration_in_words_traffic());
              }
            ?></span>
          </p>
        </div>

        <div class="quickcab-order-detail">
          <p>
            <strong class="quickcab-order-detail-label"><?php
              echo esc_html__('Departure Time', 'quickcab');
            ?></strong>
            <span><?php
              echo esc_html($order->get_departure_time_in_format());
            ?></span>
          </p>
        </div>

        <div class="quickcab-order-detail">
          <p>
            <strong class="quickcab-order-detail-label"><?php
              echo esc_html__('Arrival Time', 'quickcab') . ' (' . esc_html__('estimate', 'quickcab') . ')';;
            ?></strong>
            <span><?php
              echo esc_html($order->get_estimated_arrival_time_in_format());
            ?></span>
          </p>
        </div><?php

        if ( $order->is_return() ) {
          ?><div class="quickcab-order-detail">
            <p>
              <strong class="quickcab-order-detail-label"><?php
                echo esc_html__('Return Journey Duration', 'quickcab') . ' (' . (empty($order->get_duration_in_words_traffic()) ? esc_html__('no traffic data available', 'quickcab') : esc_html__('including traffic', 'quickcab')) . ')';
              ?></strong>
              <span><?php
                if ( empty($order->get_return_duration_in_words_traffic()) ) {
                  echo esc_html($order->get_return_duration_in_words());
                } else {
                  echo esc_html($order->get_return_duration_in_words_traffic());
                }
              ?></span>
            </p>
          </div>

          <div class="quickcab-order-detail">
            <p>
              <strong class="quickcab-order-detail-label"><?php
                echo esc_html__('Return Departure Time', 'quickcab');
              ?></strong>
              <span><?php
                echo esc_html($order->get_return_departure_time_in_format());
              ?></span>
            </p>
          </div>

          <div class="quickcab-order-detail">
            <p>
              <strong class="quickcab-order-detail-label"><?php
                echo esc_html__('Return Arrival Time', 'quickcab') . ' (' . esc_html__('estimate', 'quickcab') . ')';;
              ?></strong>
              <span><?php
                echo esc_html($order->get_estimated_return_arrival_time_in_format());
              ?></span>
            </p>
          </div><?php
        }

        if ( $booking_form->enable_passengers_field() ) {
          ?><div class="quickcab-order-detail">
            <p>
              <strong class="quickcab-order-detail-label"><?php
                echo esc_html__('Passengers', 'quickcab');
              ?></strong>
              <span><?php
                echo esc_html($order->get_occupants());
              ?></span>
            </p>
          </div><?php
        }

        if ( $booking_form->enable_suitcases_field() ) {
          ?><div class="quickcab-order-detail">
            <p>
              <strong class="quickcab-order-detail-label"><?php
                echo esc_html__('Suitcases', 'quickcab');
              ?></strong>
              <span><?php
                echo esc_html($order->get_suitcases());
              ?></span>
            </p>
          </div><?php
        }

        if ( $order->get_waiting_time() ) {
          ?><div class="quickcab-order-detail">
            <p>
              <strong class="quickcab-order-detail-label"><?php
                echo esc_html__('Waiting Time', 'quickcab');
              ?></strong>
              <span><?php
                echo esc_html($order->get_formatted_waiting_time());
              ?></span>
            </p>
          </div><?php
        }

        if ( get_post_type($order->get_form_id()) == 'qc_booking_form' ) {
          ?><div class="quickcab-order-detail">
            <p>
              <strong class="quickcab-order-detail-label"><?php
                echo esc_html__('Order Form', 'quickcab');
              ?></strong>
              <a href="<?php echo esc_attr(get_edit_post_link($order->get_form_id())); ?>" target="_blank"><?php
                echo get_the_title($order->get_form_id());
              ?></a>
            </p>
          </div><?php
        }

        foreach ( $order->get_custom_checkout_fields() as $field ) {
          if ( !empty($field['value']) ) {
            ?><div class="quickcab-order-detail">
              <p>
                <strong class="quickcab-order-detail-label"><?php
                  echo $field['label'];
                ?></strong>
                <span><?php
                  echo esc_html($field['value']);
                ?></span>
              </p>
            </div><?php
          }
        }
      ?></div>
    </div>
  </div><?php
}
add_action('quickcab_meta_box_order_details_fields', 'quickcab_render_order_details_fields');

function quickcab_order_metabox_fields() {
  return apply_filters('quickcab_metabox_order_fields_save', array(
    'quickcab_order_customer_id',
    'quickcab_order_status'
  ));
}

function quickcab_order_meta_box_save($post_id, $post) {

  if ( !isset($_POST['quickcab_order_meta_box_nonce']) || !wp_verify_nonce(wp_unslash($_POST['quickcab_order_meta_box_nonce']), basename(__FILE__)) ) {
    return;
  }

  if ( (defined('DOING_AUTOSAVE') && DOING_AUTOSAVE) || (defined('DOING_AJAX') && DOING_AJAX) || isset($_REQUEST['bulk_edit']) ) {
    return;
  }

  if ( isset($post->post_type) && 'revision' == $post->post_type ) {
    return;
  }

  $order = new QuickCab_Order($post_id);

  // The default fields that get saved
  foreach ( quickcab_order_metabox_fields() as $field ) {
    $meta_name = str_replace('quickcab_order_', '', $field);

    if ( empty(wp_unslash($_POST[$field])) ) {
      $order->delete_meta($meta_name);
    } else {
      $new = apply_filters('quickcab_order_metabox_save_' . $meta_name, wp_unslash($_POST[$field]), $post_id);
      $order->update_meta($meta_name, $new);
    }
  }

  do_action('quickcab_save_order', $post_id, $post);
}
add_action('save_post', 'quickcab_order_meta_box_save', 10, 2);
