<?php

if ( !defined('ABSPATH') ) exit;

function quickcab_surge_price_rule_columns($original_columns) {
  $new_columns = array(
    'cb'            => $original_columns['cb'],
    'title'         => esc_html__('Name', 'quickcab'),
    'dates-applied' => esc_html__('Dates/Days Applied', 'quickcab'),
    'pricing'       => esc_html__('Pricing', 'quickcab')
  );

  return apply_filters('quickcab_surge_price_rule_columns', $new_columns);
}
add_filter('manage_surge_price_rule_posts_columns', 'quickcab_surge_price_rule_columns');

function quickcab_render_surge_price_rule_columns($column_name, $post_id) {
  if ( get_post_type($post_id) == 'surge_price_rule' ) {
    $surge_price_rule = new QuickCab_Surge_Price_Rule($post_id);

    switch ( $column_name ) {
      case 'dates-applied':

        $days = array(
          1 => esc_html__('Monday', 'quickcab'),
          2 => esc_html__('Tuesday', 'quickcab'),
          3 => esc_html__('Wednesday', 'quickcab'),
          4 => esc_html__('Thursday', 'quickcab'),
          5 => esc_html__('Friday', 'quickcab'),
          6 => esc_html__('Saturday', 'quickcab'),
          0 => esc_html__('Sunday', 'quickcab')
        );

        if ( $surge_price_rule->uses_fixed_dates() ) {
          $start_date = $surge_price_rule->get_end_date_timestamp();
          $end_date = $surge_price_rule->get_end_date_timestamp();
          $format = get_option('date_format') . ' ' . get_option('time_format');

          ?><strong><?php echo esc_html__('From', 'quickcab'); ?>:</strong> <?php echo date($format, $start_date); ?><br />
          <strong><?php echo esc_html__('To', 'quickcab'); ?>:</strong> <?php echo date($format, $end_date);
        } else if ( $surge_price_rule->uses_days_of_week() ) {
          ?><table>
            <thead>
              <tr>
                <th><?php echo esc_html__('From'); ?></th>
                <th><?php echo esc_html__('To'); ?></th>
              </tr>
            </thead>
            <tbody><?php
              foreach ( $surge_price_rule->get_day_tiers() as $tier ) {
                ?><tr>
                  <td><?php
                    echo esc_html($days[$tier['from_day']]) . ' ' . esc_html__('at', 'quickcab') . ' ' . sprintf('%02d', $tier['from_hour']) . ':' . sprintf('%02d', $tier['from_min']);
                  ?></td>
                  <td><?php
                    echo esc_html($days[$tier['to_day']]) . ' ' . esc_html__('at', 'quickcab') . ' ' . sprintf('%02d', $tier['to_hour']) . ':' . sprintf('%02d', $tier['to_min']);
                  ?></td>
                </tr><?php
              }
            ?></tbody>
          </table><?php
        }

        break;
      case 'pricing':
        if ( $surge_price_rule->is_increase_amount() ) {
          echo esc_html__('Amount', 'quickcab') . ': +' . quickcab_format_price($surge_price_rule->get_increase_amount());
        } else if ( $surge_price_rule->is_increase_multiplier() ) {
          echo esc_html__('Multiplier', 'quickcab') . ': x' . $surge_price_rule->get_increase_multiplier();
        }
        break;
    }
  }
}
add_action('manage_posts_custom_column', 'quickcab_render_surge_price_rule_columns', 10, 2);
