<?php

if ( !defined('ABSPATH') ) exit;

class QuickCab_Geofence_Shape_Coord_DB extends QuickCab_DB {

  public function __construct() {
    global $wpdb;
    $this->table_name = $wpdb->prefix . 'quickcab_geofence_shape_coordinates';
  }

  public function get_by($field, $value) {
    if ( $field == 'id' || $field = 'geofence_shape_id' ) {
      if ( !is_numeric($value) ) {
        return false;
      }

      $value = intval($value);

      if ( $value < 1 ) {
        return false;
      }
    }

    return parent::get_by($field, $value);
  }

  public function get_geofence_shape_coords($args = array()) {
    global $wpdb;

    if ( empty($args) ) {
      return false;
    }

    $defaults = array(
      'order_by' => 'id',
      'order'    => 'ASC'
    );

    $args = wp_parse_args($args, $defaults);
    $where = ' WHERE 1=1 ';
    $order = $args['order'];
    $orderby = $args['order_by'];

    unset($args['order_by']);
    unset($args['order']);

    if ( !empty($args['id']) ) {
      if ( is_array($args['id']) ) {
        $ids = implode(',', array_map('intval', $args['id']));
      } else {
        $ids = intval($args['id']);
      }

      $where .= " AND `id` IN({$ids}) ";
      unset($args['id']);
    }

    if ( !empty($args['geofence_rule_id']) ) {
      if ( is_array($args['geofence_rule_id']) ) {
        $user_ids = implode(',', array_map('intval', $args['geofence_rule_id']));
      } else {
        $user_ids = intval($args['geofence_rule_id']);
      }

      $where .= " AND `geofence_rule_id` IN({$user_ids}) ";
      unset($args['geofence_rule_id']);
    }

    // any attributes that dont need special attention just use =
    foreach ( $args as $key => $val ) {
      $where .= " AND `" . esc_sql($key) . "` = '" . esc_sql($val) . "' ";
    }

    return $wpdb->get_results(
      "SELECT * FROM `" . esc_sql($this->table_name) . "` $where ORDER BY {$orderby} {$order};"
    );
  }

  public function add($data = array()) {
    global $wpdb;

    if ( isset($data['id']) && !empty($data['id']) ) {
      if ( !is_numeric($data['id']) ) {
        return false;
      }

      $this->update($data['id'], $data);

      return $data['id'];
    }

    return $this->insert($data);
  }

  public function delete($id = false) {
    if ( empty($id) || !is_numeric($id) ) {
      return false;
    }

    $geofence_shape_coord = $this->get_by('id', $id);

    if ( $geofence_shape_coord ) {
      global $wpdb;
      return $wpdb->delete($this->table_name, array('id' => $id), array('%d'));
    }

    return false;
  }

  public function delete_by_shape_id($shape_id = false) {
    global $wpdb;

    if ( empty($shape_id) || !is_numeric($shape_id) ) {
      return false;
    }

    return $wpdb->delete(
      $this->table_name,
      array('geofence_shape_id' => $shape_id),
      array('%d')
    );
  }
}
