<?php

if ( !defined('ABSPATH') ) exit;

class QuickCab_Geofence_Shape_Coord {

  protected $id;
  protected $geofence_shape_id;
  protected $latitude;
  protected $longitude;

  protected $db;

  public function __construct($value = false, $field = 'id') {
    $this->db = new QuickCab_Geofence_Shape_Coord_DB;

    if ( $value === false ) {
      return false;
    }

    $geofence_shape_coord = $this->db->get_by($field, $value);

    if ( empty($geofence_shape_coord) ) {
      return false;
    }

    $this->setup_geofence_shape_coord($geofence_shape_coord);
  }

  public function get_id() {
    return $this->id;
  }

  public function get_geofence_shape_id() {
    return (int) apply_filters('quickcab_note_get_geofence_shape_id', $this->geofence_shape_id, $this->get_id());
  }

  public function get_geofence_shape() {
    return new QuickCab_Geofence_Shape($this->get_geofence_shape_id());
  }

  public function get_latitude() {
    return apply_filters('quickcab_note_get_latitude', $this->latitude, $this->get_id());
  }

  public function get_longitude() {
    return apply_filters('quickcab_note_get_longitude', $this->longitude, $this->get_id());
  }

  public function create($data = array()) {
    if ( empty($data['geofence_shape_id']) || empty($data['latitude']) || empty($data['longitude']) ) {
      return false;
    }

    do_action('quickcab_before_create_geofence_shape_coord', $data);

    $id = false;

    if ( $geofence_shape_coord_id = $this->db->add($data) ) {
      $geofence_shape_coord = $this->db->get_by('id', $geofence_shape_coord_id);

      $this->setup_geofence_shape_coord($geofence_shape_coord);

      $id = $geofence_shape_coord_id;
    }

    do_action('quickcab_after_create_geofence_shape_coord', $id, $data);

    return $id;
  }

  /**
   * Assigns the variables of the class
   *
   * @since  1.0
   * @param  object  $geofence_shape_coord  the object returned by the wpdb query
   * @return boolean                        did it succeed?
   */
  private function setup_geofence_shape_coord($geofence_shape_coord) {
    if ( !is_object($geofence_shape_coord) ) {
      return false;
    }

    foreach ( $geofence_shape_coord as $key => $value ) {
      $this->$key = $value;
    }

    if ( empty($this->get_id()) || empty($this->get_geofence_shape_id()) || empty($this->get_latitude()) || empty($this->get_longitude()) ) {
      return false;
    }

    return true;
  }
}
