<?php

if ( !defined('ABSPATH') ) exit;

class QuickCab_Geofence_Shape_DB extends QuickCab_DB {

  public function __construct() {
    global $wpdb;
    $this->table_name = $wpdb->prefix . 'quickcab_geofence_shapes';
  }

  public function get_by($field, $value) {
    if ( $field == 'id' ) {
      if ( !is_numeric($value) ) {
        return false;
      }

      $value = intval($value);

      if ( $value < 1 ) {
        return false;
      }
    }

    return parent::get_by($field, $value);
  }

  public function get_geofence_shapes($args = array()) {
    global $wpdb;

    if ( empty($args) ) {
      return false;
    }

    $defaults = array(
      'order_by' => 'date_created',
      'order'    => 'ASC'
    );

    $args = wp_parse_args($args, $defaults);
    $where = ' WHERE 1=1 ';
    $order = $args['order'];
    $orderby = $args['order_by'];
    unset($args['order_by']);
    unset($args['order']);

    if ( !empty($args['id']) ) {
      if ( is_array($args['id']) ) {
        $ids = implode(',', array_map('intval', $args['id']));
      } else {
        $ids = intval($args['id']);
      }

      $where .= " AND `id` IN({$ids}) ";
      unset($args['id']);
    }

    // any attributes that dont need special attention just use =
    foreach ( $args as $key => $val ) {
      $where .= " AND `" . esc_sql($key) . "` = '" . esc_sql($val) . "' ";
    }

    return $wpdb->get_results(
      "SELECT * FROM `" . esc_sql($this->table_name) . "` $where ORDER BY {$orderby} {$order};"
    );
  }

  public function add($data = array()) {
    global $wpdb;

    if ( isset($data['id']) && !empty($data['id']) ) {
      if ( !is_numeric($data['id']) ) {
        return false;
      }

      $this->update($data['id'], $data);

      return $data['id'];
    }

    return $this->insert($data);
  }

  public function delete($id = false) {
    if ( empty($id) || !is_numeric($id) ) {
      return false;
    }

    $geofence_shape = $this->get_by('id', $id);

    if ( $geofence_shape ) {
      global $wpdb;
      return $wpdb->delete($this->table_name, array('id' => $id), array('%d'));
    }

    return false;
  }

  public function find_all_available() {
    global $wpdb;

    return $wpdb->get_results(
      "SELECT * FROM `" . esc_sql($this->table_name) . "` WHERE `name` IS NOT NULL;"
    );
  }

}
