<?php

if ( !defined('ABSPATH') ) exit;

function quickcab_create_databases() {
  require_once ABSPATH . 'wp-admin/includes/upgrade.php';

  global $wpdb;
  global $charset_collate;

  $tiered_pricing_tbl        = $wpdb->prefix . 'quickcab_tiered_pricing';
  $customers_tbl             = $wpdb->prefix . 'quickcab_customers';
  $notes_tbl                 = $wpdb->prefix . 'quickcab_notes';
  $geofence_shape_tbl        = $wpdb->prefix . 'quickcab_geofence_shapes';
  $geofence_shape_coords_tbl = $wpdb->prefix . 'quickcab_geofence_shape_coordinates';
  $charset_collate           = $wpdb->get_charset_collate();

  // Create Vehicles Table
  $sql_tiered_pricing = "CREATE TABLE IF NOT EXISTS `$tiered_pricing_tbl` (
    `id` int(11) unsigned NOT NULL auto_increment,
    `vehicle_id` int(11) NOT NULL,
    `value_from` int(11) NOT NULL,
    `value_to` int(11) NOT NULL,
    `price` numeric(15, 2) NOT NULL default 0.00,
    `type` varchar(255) NOT NULL,
    PRIMARY KEY (id)
  ) $charset_collate;";

  dbDelta($sql_tiered_pricing);

  // Create Customers Table
  $sql_customers = "CREATE TABLE IF NOT EXISTS `$customers_tbl` (
    `id` int(11) unsigned NOT NULL auto_increment,
    `user_id` int(11) NOT NULL,
    `first_name` varchar(255) NOT NULL,
    `last_name` varchar(255) NOT NULL,
    `email` varchar(255) NOT NULL,
    `address1` varchar(255) NOT NULL,
    `address2` varchar(255) NOT NULL,
    `town_city` varchar(255) NOT NULL,
    `zip` varchar(32) NOT NULL,
    `country` varchar(255) NOT NULL,
    `purchase_ids` longtext NOT NULL,
    `date_created` datetime NOT NULL,
    PRIMARY KEY (id)
  ) $charset_collate;";

  dbDelta($sql_customers);

  // Create Notes Table
  $sql_notes = "CREATE TABLE IF NOT EXISTS `$notes_tbl` (
    `id` int(11) unsigned NOT NULL auto_increment,
    `item_type` varchar(255) NOT NULL,
    `item_id` int(11) NOT NULL,
    `text` longtext NOT NULL,
    `user_id` int(11) NOT NULL,
    `date_created` datetime NOT NULL,
    PRIMARY KEY (id)
  ) $charset_collate;";

  dbDelta($sql_notes);

  $sql_geofence_shape_table = "CREATE TABLE IF NOT EXISTS `$geofence_shape_tbl` (
    `id` int(11) NOT NULL AUTO_INCREMENT,
    `name` varchar(255) NOT NULL,
    PRIMARY KEY (`id`)
  ) $charset_collate;";

  dbDelta($sql_geofence_shape_table);

  // Create Geofence Shape Coordinates Table
  $sql_geofence_shape_coords_table = "CREATE TABLE IF NOT EXISTS `$geofence_shape_coords_tbl` (
    `id` int(11) NOT NULL AUTO_INCREMENT,
    `geofence_shape_id` int(11) NOT NULL,
    `latitude` varchar(255) NOT NULL,
    `longitude` varchar(255) NOT NULL,
    PRIMARY KEY (`id`)
  ) $charset_collate;";

  dbDelta($sql_geofence_shape_coords_table);
}

/**
 * Will return the following statement
 *    $installed_version ($operator) $new_version
 * OR returns true if $installed_version doesn't exist
 */
function quickcab_installed_version_check($new_version, $operator) {
  $installed_version = quickcab_get_option('quickcab_version');

  // if no previous version found then return true
  if ( !$installed_version ) {
    return true;
  }

  return version_compare($installed_version, $new_version, $operator);
}

function quickcab_install_data_1_2_4() {
  // populate all booking fields by default to show all options
  $booking_forms = QuickCab_Booking_Form::find_all();

  foreach ( $booking_forms as $booking_form ) {
    $booking_form->update_meta('show_distance', true);
    $booking_form->update_meta('show_duration', true);
    $booking_form->update_meta('enable_passengers_field', true);
    $booking_form->update_meta('enable_suitcases_field', true);
    $booking_form->update_meta('enable_direction_field', true);
  }
}

function quickcab_install_required_data() {
  $data_install_function = array(
    '1.2.4' => 'quickcab_install_data_1_2_4'
  );

  foreach ( $data_install_function as $version => $func ) {
    if ( quickcab_installed_version_check($version, '>') ) {
      call_user_func($func);
    }
  }
}

function quickcab_install_custom_database() {
  quickcab_create_databases();
  quickcab_install_required_data();

  quickcab_set_option('quickcab_version', QUICKCAB_VERSION);
}
