<?php

if ( !defined('ABSPATH') ) exit;

function quickcab_render_surge_price_metabox_price_row($name, $post_id, $label_options = array(), $input_options = array(), $helper_options = array()) {
  $meta_name = str_replace('quickcab_surge_price_', '', $name);
  $surge_price_rule = new QuickCab_Surge_Price_Rule($post_id);

  if ( !isset($input_options['value']) ) {
    $input_options['value'] = $surge_price_rule->get_meta($meta_name);
  }

  quickcab_render_metabox_price_row($name, $post_id, $label_options, $input_options, $helper_options);
}

function quickcab_render_surge_price_metabox_number_row($name, $post_id, $label_options = array(), $input_options = array(), $helper_options = array()) {
  $meta_name = str_replace('quickcab_surge_price_', '', $name);
  $surge_price_rule = new QuickCab_Surge_Price_Rule($post_id);

  if ( !isset($input_options['value']) ) {
    $input_options['value'] = $surge_price_rule->get_meta($meta_name);
  }

  quickcab_render_metabox_number_row($name, $post_id, $label_options, $input_options, $helper_options);
}

/**
 * Adds the surge price fee to the price
 *
 * @since  1.0
 * @param  array       $price_info  the current $price_info array
 * @param  QuickCab_Vehicle  $vehicle     the vehicle being quoted
 * @param  QuickCab_Journey  $journey     the journey being quoted
 * @return array                    $price_info
 */
function quickcab_add_surge_price_to_fare($price_info, $vehicle, $journey) {
  $surge_price_rules = QuickCab_Surge_Price_Rule::find_all();

  foreach ( $surge_price_rules as $surge_price_rule ) {
    if ( $surge_price_rule->applies_to($journey) ) {
      if ( $surge_price_rule->is_increase_amount() ) {
        $price_info = quikcab_add_to_price_info(
          $price_info,
          'surge_pricing',
          esc_html__('Surge Pricing', 'quickcab'),
          $surge_price_rule->get_increase_amount(),
          ''
        );
      } else if ( $surge_price_rule->is_increase_multiplier() ) {
        $multiplier = $surge_price_rule->get_increase_multiplier() - 1;

        $price_info = quikcab_add_to_price_info(
          $price_info,
          'surge_pricing',
          esc_html__('Surge Pricing', 'quickcab'),
          $multiplier * $price_info['total_unformatted'],
          ''
        );
      }
    }
  }

  return $price_info;
}
add_filter('quickcab_price_after_all_vehicle_costs_calculation', 'quickcab_add_surge_price_to_fare', 12, 3);

/**
 * Gives the minutes since midnight
 *
 * @since   1.0
 * @param   int     $time  Unix Epoch time
 * @return  int            number of minutes since midnight
 */
function quickcab_mins_since_midnight($time) {
  return intval(date('G', $time)) * 60 + intval(date('i', $time));
}

/**
 * Is the day of week in the given range of day (uses PHPs date('w') to
 * determine the day of week). Accounts for overflow (i.e. day being over 6)
 *
 * @since   1.0
 * @param   int     $dow  the day of Week
 * @param   int     $min  the min day
 * @param   int     $max  the max day
 * @return  boolean       is it in the range?
 */
function quickcab_day_in_day_range($dow, $min, $max) {
  if ( $max < $min ) {
    return $dow <= $max || $dow >= $min;
  }

  return $dow >= $min && $dow <= $max;
}
