<?php

if ( !defined('ABSPATH') ) exit;

function quickcab_plugin_url($path = '') {
  $url = plugins_url($path, QUICKCAB_PLUGIN);

  if ( is_ssl() && 'http:' == substr($url, 0, 5) ) {
    $url = 'https:' . substr($url, 5);
  }

  return $url;
}

/**
 * A list of countries and their codes in ISO3166 format
 *
 * @since  1.0
 * @return array  list of countries and their ISO3166 codes
 */
function quickcab_iso_3166_country_codes() {
  return array(
    'AF' => 'Afghanistan', 'AX' => 'Aland Islands', 'AL' => 'Albania', 'DZ' => 'Algeria', 'AS' => 'American Samoa', 'AD' => 'Andorra', 'AO' => 'Angola', 'AI' => 'Anguilla', 'AQ' => 'Antarctica', 'AG' => 'Antigua And Barbuda', 'AR' => 'Argentina', 'AM' => 'Armenia', 'AW' => 'Aruba', 'AU' => 'Australia', 'AT' => 'Austria', 'AZ' => 'Azerbaijan', 'BS' => 'Bahamas', 'BH' => 'Bahrain', 'BD' => 'Bangladesh', 'BB' => 'Barbados', 'BY' => 'Belarus', 'BE' => 'Belgium', 'BZ' => 'Belize', 'BJ' => 'Benin', 'BM' => 'Bermuda', 'BT' => 'Bhutan', 'BO' => 'Bolivia', 'BA' => 'Bosnia And Herzegovina', 'BW' => 'Botswana', 'BV' => 'Bouvet Island', 'BR' => 'Brazil', 'IO' => 'British Indian Ocean Territory', 'BN' => 'Brunei Darussalam', 'BG' => 'Bulgaria', 'BF' => 'Burkina Faso', 'BI' => 'Burundi', 'KH' => 'Cambodia', 'CM' => 'Cameroon', 'CA' => 'Canada', 'CV' => 'Cape Verde', 'KY' => 'Cayman Islands', 'CF' => 'Central African Republic', 'TD' => 'Chad', 'CL' => 'Chile', 'CN' => 'China', 'CX' => 'Christmas Island', 'CC' => 'Cocos (Keeling) Islands', 'CO' => 'Colombia', 'KM' => 'Comoros', 'CG' => 'Congo', 'CD' => 'Congo, Democratic Republic', 'CK' => 'Cook Islands', 'CR' => 'Costa Rica', 'CI' => 'Cote D\'Ivoire', 'HR' => 'Croatia', 'CU' => 'Cuba', 'CY' => 'Cyprus', 'CZ' => 'Czech Republic', 'DK' => 'Denmark', 'DJ' => 'Djibouti', 'DM' => 'Dominica', 'DO' => 'Dominican Republic', 'EC' => 'Ecuador', 'EG' => 'Egypt', 'SV' => 'El Salvador', 'GQ' => 'Equatorial Guinea', 'ER' => 'Eritrea', 'EE' => 'Estonia', 'ET' => 'Ethiopia', 'FK' => 'Falkland Islands (Malvinas)', 'FO' => 'Faroe Islands', 'FJ' => 'Fiji', 'FI' => 'Finland', 'FR' => 'France', 'GF' => 'French Guiana', 'PF' => 'French Polynesia', 'TF' => 'French Southern Territories', 'GA' => 'Gabon', 'GM' => 'Gambia', 'GE' => 'Georgia', 'DE' => 'Germany', 'GH' => 'Ghana', 'GI' => 'Gibraltar', 'GR' => 'Greece', 'GL' => 'Greenland', 'GD' => 'Grenada', 'GP' => 'Guadeloupe', 'GU' => 'Guam', 'GT' => 'Guatemala', 'GG' => 'Guernsey', 'GN' => 'Guinea', 'GW' => 'Guinea-Bissau', 'GY' => 'Guyana', 'HT' => 'Haiti', 'HM' => 'Heard Island & Mcdonald Islands', 'VA' => 'Holy See (Vatican City State)', 'HN' => 'Honduras', 'HK' => 'Hong Kong', 'HU' => 'Hungary', 'IS' => 'Iceland', 'IN' => 'India', 'ID' => 'Indonesia', 'IR' => 'Iran, Islamic Republic Of', 'IQ' => 'Iraq', 'IE' => 'Ireland', 'IM' => 'Isle Of Man', 'IL' => 'Israel', 'IT' => 'Italy', 'JM' => 'Jamaica', 'JP' => 'Japan', 'JE' => 'Jersey', 'JO' => 'Jordan', 'KZ' => 'Kazakhstan', 'KE' => 'Kenya', 'KI' => 'Kiribati', 'KR' => 'Korea', 'KW' => 'Kuwait', 'KG' => 'Kyrgyzstan', 'LA' => 'Lao People\'s Democratic Republic', 'LV' => 'Latvia', 'LB' => 'Lebanon', 'LS' => 'Lesotho', 'LR' => 'Liberia', 'LY' => 'Libyan Arab Jamahiriya', 'LI' => 'Liechtenstein', 'LT' => 'Lithuania', 'LU' => 'Luxembourg', 'MO' => 'Macao', 'MK' => 'Macedonia', 'MG' => 'Madagascar', 'MW' => 'Malawi', 'MY' => 'Malaysia', 'MV' => 'Maldives', 'ML' => 'Mali', 'MT' => 'Malta', 'MH' => 'Marshall Islands', 'MQ' => 'Martinique', 'MR' => 'Mauritania', 'MU' => 'Mauritius', 'YT' => 'Mayotte', 'MX' => 'Mexico', 'FM' => 'Micronesia, Federated States Of', 'MD' => 'Moldova', 'MC' => 'Monaco', 'MN' => 'Mongolia', 'ME' => 'Montenegro', 'MS' => 'Montserrat', 'MA' => 'Morocco', 'MZ' => 'Mozambique', 'MM' => 'Myanmar', 'NA' => 'Namibia', 'NR' => 'Nauru', 'NP' => 'Nepal', 'NL' => 'Netherlands', 'AN' => 'Netherlands Antilles', 'NC' => 'New Caledonia', 'NZ' => 'New Zealand', 'NI' => 'Nicaragua', 'NE' => 'Niger', 'NG' => 'Nigeria', 'NU' => 'Niue', 'NF' => 'Norfolk Island', 'MP' => 'Northern Mariana Islands', 'NO' => 'Norway', 'OM' => 'Oman', 'PK' => 'Pakistan', 'PW' => 'Palau', 'PS' => 'Palestinian Territory, Occupied', 'PA' => 'Panama', 'PG' => 'Papua New Guinea', 'PY' => 'Paraguay', 'PE' => 'Peru', 'PH' => 'Philippines', 'PN' => 'Pitcairn', 'PL' => 'Poland', 'PT' => 'Portugal', 'PR' => 'Puerto Rico', 'QA' => 'Qatar', 'RE' => 'Reunion', 'RO' => 'Romania', 'RU' => 'Russian Federation', 'RW' => 'Rwanda', 'BL' => 'Saint Barthelemy', 'SH' => 'Saint Helena', 'KN' => 'Saint Kitts And Nevis', 'LC' => 'Saint Lucia', 'MF' => 'Saint Martin', 'PM' => 'Saint Pierre And Miquelon', 'VC' => 'Saint Vincent And Grenadines', 'WS' => 'Samoa', 'SM' => 'San Marino', 'ST' => 'Sao Tome And Principe', 'SA' => 'Saudi Arabia', 'SN' => 'Senegal', 'RS' => 'Serbia', 'SC' => 'Seychelles', 'SL' => 'Sierra Leone', 'SG' => 'Singapore', 'SK' => 'Slovakia', 'SI' => 'Slovenia', 'SB' => 'Solomon Islands', 'SO' => 'Somalia', 'ZA' => 'South Africa', 'GS' => 'South Georgia And Sandwich Isl.', 'ES' => 'Spain', 'LK' => 'Sri Lanka', 'SD' => 'Sudan', 'SR' => 'Suriname', 'SJ' => 'Svalbard And Jan Mayen', 'SZ' => 'Swaziland', 'SE' => 'Sweden', 'CH' => 'Switzerland', 'SY' => 'Syrian Arab Republic', 'TW' => 'Taiwan', 'TJ' => 'Tajikistan', 'TZ' => 'Tanzania', 'TH' => 'Thailand', 'TL' => 'Timor-Leste', 'TG' => 'Togo', 'TK' => 'Tokelau', 'TO' => 'Tonga', 'TT' => 'Trinidad And Tobago', 'TN' => 'Tunisia', 'TR' => 'Turkey', 'TM' => 'Turkmenistan', 'TC' => 'Turks And Caicos Islands', 'TV' => 'Tuvalu', 'UG' => 'Uganda', 'UA' => 'Ukraine', 'AE' => 'United Arab Emirates', 'GB' => 'United Kingdom', 'US' => 'United States', 'UY' => 'Uruguay', 'UZ' => 'Uzbekistan', 'VU' => 'Vanuatu', 'VE' => 'Venezuela', 'VN' => 'Viet Nam', 'VG' => 'Virgin Islands, British', 'VI' => 'Virgin Islands, U.S.', 'WF' => 'Wallis And Futuna', 'EH' => 'Western Sahara', 'YE' => 'Yemen', 'ZM' => 'Zambia', 'ZW' => 'Zimbabwe',
  );
}

/**
 * Render the subnav for the settings pages
 *
 * @since  1.0
 * @param  array  $tabs  the settings tabs
 * @return void
 */
function quickcab_render_tab_sub_nav($tabs) {
  ?><ul class="subsubsub"><?php

    for ( $i = 0; $i < count($tabs); $i++ ) {
      $tab = $tabs[$i];

      ?><li>
        <a <?php if ($tab['current']) { ?>class="current"<?php } ?> href="<?php echo esc_attr($tab['link']); ?>"><?php echo esc_html($tab['name']); ?></a><?php

        if ( $i < count($tabs) - 1 ) {
          ?> |<?php
        }
      ?></li><?php
    }
  ?></ul><?php
}

/**
 * Check if the current page is a post page
 *
 * @since  1.0
 * @param  string $postname  the post type
 * @param  string $pagename  [edit|new]
 * @return bool
 */
function quickcab_on_post_page($postname, $pagename = null) {
  global $pagenow;
  global $typenow;

  if ( is_null($pagename) ) {
    return in_array($pagenow, array('post.php', 'post-new.php')) && $typenow == $postname;
  } else {
    if ( $pagename == 'edit' ) {
      return in_array($pagenow, array('post.php')) && $typenow == $postname;
    } else if ( $pagename == 'new' ) {
      return in_array($pagenow, array('post-new.php')) && $typenow == $postname;
    }
  }

  return false;
}

/**
 * Formats the price using wc_price()
 *
 * @since  1.0
 * @param  float  $price   the price to be formatted
 * @param  array  $options options array
 * @return string          formatted price
 */
function quickcab_format_price($price, $options = array()) {
  return apply_filters('quickcab_formatted_price', wc_price($price, $options));
}

function quickcab_register_image_sizes() {
  add_image_size('small-thumb', 170);
  add_image_size('smaller-thumb', 110);
}
add_action('init', 'quickcab_register_image_sizes');

/**
 * Add an item to the given price_info
 *
 * @since  1.0
 * @param  array  $price_info  the price info array to be added to
 * @param  string $key         the unique key of the item to be added (snake_case)
 * @param  string $title       the title of the item, this will show in the price breakdown
 * @param  float  $amount      the amount of the item
 * @param  string $rate        the rate of the charge (description given to user)
 * @return array               the price info array with new item added
 */
function quikcab_add_to_price_info($price_info, $key, $title, $amount, $rate) {
  if ( $amount == 0 ) {
    return $price_info;
  }

  $round = intval(quickcab_get_option('decimal_points_shown'));
  $amount = round($amount, $round);
  $amount_after_tax = $amount;

  if ( array_key_exists($key, $price_info['breakdown']) ) {
    $price_info['breakdown'][$key]['amount_unformatted'] += $amount_after_tax;
    $price_info['breakdown'][$key]['amount_formatted'] = quickcab_format_price($price_info['breakdown'][$key]['amount_unformatted']);
  } else {
    $price_info['breakdown'][$key] = apply_filters('quickcab_add_price_breakdown', array(
      'title'              => $title,
      'amount_unformatted' => $amount_after_tax,
      'amount_formatted'   => quickcab_format_price($amount_after_tax),
      'rate'               => $rate
    ));
  }

  // Recalculate total
  $price_info['total_unformatted'] = 0;

  foreach ( $price_info['breakdown'] as $key => $data ) {
    $price_info['total_unformatted'] += $data['amount_unformatted'];
  }

  $price_info['total_formatted'] = quickcab_format_price($price_info['total_unformatted']);

  return $price_info;
}

/**
 * Merge 2 price_info arrays
 *
 * @since  1.0
 * @param  array $price_info1 price info array
 * @param  array $price_info2 price info array
 * @return array              merged price info array
 */
function quickcab_merge_price_info($price_info1, $price_info2) {
  foreach ( $price_info2['breakdown'] as $key => $data ) {
    $price_info1 = quikcab_add_to_price_info(
      $price_info1,
      $key,
      $data['title'],
      $data['amount_unformatted'],
      $data['rate']
    );
  }

  return $price_info1;
}

/**
 * Get the corresponding google maps code for the WordPress locale
 *
 * @since  1.0
 * @return string  the google maps locale code
 */
function get_google_maps_locale_code() {
  $wp_locale = get_locale();

  switch ( $wp_locale ) {
    case 'bg_BG':
      return 'bg';
    case 'bn_BD':
      return 'bn';
    case 'cs_CZ':
      return 'cs';
    case 'da_DK':
      return 'da';
    case 'de_DE':
    case 'de_CH':
      return 'de';
    case 'en_US':
      return 'en';
    case 'en_AU':
      return 'en-AU';
    case 'en_GB':
      return 'en-GB';
    case 'es_ES':
      return 'es';
    case 'fa_IR':
      return 'fa';
    case 'fr_FR':
    case 'fr_BE':
      return 'fr';
    case 'gl_ES':
      return 'gl';
    case 'gu_IN':
      return 'gu';
    case 'hi_IN':
      return 'hi';
    case 'hu_HU':
      return 'hu';
    case 'id_ID':
      return 'id';
    case 'it_IT':
      return 'it';
    case 'he_IL':
      return 'iw';
    case 'kn':
      return 'kn';
    case 'ko_KR':
      return 'ko';
    case 'lt_LT':
      return 'lt';
    case 'ml_IN':
      return 'ml';
    case 'nl_NL':
    case 'nl_BE':
      return 'nl';
    case 'pl_PL':
      return 'pl';
    case 'pt_PT':
      return 'pt';
    case 'pt_BR':
      return 'pt-BR';
    case 'pt_PT':
      return 'pt-PT';
    case 'ro_RO':
      return 'ro';
    case 'ru_RU':
      return 'ru';
    case 'sk_SK':
      return 'sk';
    case 'sl_SI':
      return 'sl';
    case 'sr_RS':
      return 'sr';
    case 'sv_SE':
      return 'sv';
    case 'ta_IN':
      return 'ta';
    case 'tr_TR':
      return 'tr';
    case 'zh_CN':
      return 'zh-CN';
    case 'zh_TW':
      return 'zh-TW';
    default:
      return 'en';
  }
}

/**
 * Get the corresponding fullcalendar code for the WordPress locale
 *
 * @since  1.0
 * @return string  the fullcalendar locale code
 */
function get_fullcalendar_locale_code() {
  $wp_locale = get_locale();

  switch ( $wp_locale ) {
    case 'af':
      return 'af';
    case 'ar':
      return 'ar';
    case 'bg_BG':
      return 'bg';
    case 'bs_BA':
      return 'bs';
    case 'ca':
      return 'ca';
    case 'cs_CZ':
      return 'cs';
    case 'da_DK':
      return 'da';
    case 'de_DE':
    case 'de_CH':
      return 'de';
    case 'el':
      return 'el';
    case 'en_US':
      return 'en-us';
    case 'en_AU':
      return 'en-au';
    case 'en_GB':
      return 'en-gb';
    case 'es_ES':
    case 'es_AR':
    case 'es_CL':
    case 'es_CO':
    case 'es_MX':
    case 'es_PE':
    case 'es_PR':
    case 'es_VE':
      return 'es';
    case 'et':
      return 'et';
    case 'eu':
      return 'eu';
    case 'fa_IR':
      return 'fa';
    case 'fr_FR':
    case 'fr_BE':
      return 'fr';
    case 'gl_ES':
      return 'gl';
    case 'hi_IN':
      return 'hi';
    case 'hu_HU':
      return 'hu';
    case 'id_ID':
      return 'id';
    case 'it_IT':
      return 'it';
    case 'ko_KR':
      return 'ko';
    case 'lt_LT':
      return 'lt';
    case 'nb_NO':
      return 'nb';
    case 'nl_NL':
    case 'nl_BE':
      return 'nl';
    case 'nn_NO':
      return 'nn';
    case 'pl_PL':
      return 'pl';
    case 'pt_PT':
      return 'pt';
    case 'pt_BR':
      return 'pt-br';
    case 'ro_RO':
      return 'ro';
    case 'ru_RU':
      return 'ru';
    case 'sk_SK':
      return 'sk';
    case 'sl_SI':
      return 'sl';
    case 'sq':
      return 'sq';
    case 'sr_RS':
      return 'sr';
    case 'sv_SE':
      return 'sv';
    case 'tr_TR':
      return 'tr';
    case 'vi':
      return 'vi';
    case 'zh_CN':
      return 'zh-cn';
    case 'zh_TW':
      return 'zh-tw';
    default:
      return 'en-us';
  }
}

/**
 * Changes the brightness of a hex color
 *
 * @since  1.0
 * @link   https://stackoverflow.com/questions/3512311/how-to-generate-lighter-darker-color-with-php
 * @param  string $hex    the hex color
 * @param  int    $steps  steps should be between -255 and 255. negative = darker, positive = lighter
 * @return string         new hex color
 */
function quickcab_adjust_hex_brightness($hex, $steps) {
  $steps = max(-255, min(255, $steps));

  // Normalize into a six character long hex string
  $hex = str_replace('#', '', $hex);
  if ( strlen($hex) == 3 ) {
    $hex = str_repeat(substr($hex, 0, 1), 2) . str_repeat(substr($hex, 1, 1), 2) . str_repeat(substr($hex, 2, 1), 2);
  }

  // Split into three parts: R, G and B
  $color_parts = str_split($hex, 2);
  $return = '#';

  foreach ( $color_parts as $color ) {
    $color   = hexdec($color); // Convert to decimal
    $color   = max(0, min(255, $color + $steps)); // Adjust color
    $return .= str_pad(dechex($color), 2, '0', STR_PAD_LEFT); // Make two char hex code
  }

  return $return;
}

/**
 * Return a random hexadecimal color
 *
 * @since  1.1.0
 * @return string the random color
 */
function quickcab_random_color() {
  return '#' . str_pad(dechex(mt_rand(0, 0xFFFFFF)), 6, '0', STR_PAD_LEFT);
}

function quickcab_get_text_color($hex_color) {
  // hex_color rgb
  $r1 = hexdec(substr($hex_color, 1, 2));
  $g1 = hexdec(substr($hex_color, 3, 2));
  $b1 = hexdec(substr($hex_color, 5, 2));

  // Black RGB
  $black_color = "#000000";
  $r2_black_color = hexdec(substr($black_color, 1, 2));
  $g2_black_color = hexdec(substr($black_color, 3, 2));
  $b2_black_color = hexdec(substr($black_color, 5, 2));

  // Calc contrast ratio
  $l1 = 0.2126 * pow($r1 / 255, 2.2) +
        0.7152 * pow($g1 / 255, 2.2) +
        0.0722 * pow($b1 / 255, 2.2);

  $l2 = 0.2126 * pow($r2_black_color / 255, 2.2) +
        0.7152 * pow($g2_black_color / 255, 2.2) +
        0.0722 * pow($b2_black_color / 255, 2.2);

  $contrast_ratio = 0;

  if ($l1 > $l2) {
    $contrast_ratio = intval(($l1 + 0.05) / ($l2 + 0.05));
  } else {
    $contrast_ratio = intval(($l2 + 0.05) / ($l1 + 0.05));
  }

  // if contrast is more than 5, return black color
  if ( $contrast_ratio > 5 ) {
    return '#000';
  } else { 
    return '#fff';
  }
}

/**
 * Get the singular name for the current units
 *
 * @since  1.0
 * @param  string  $unit  the current units
 * @return string         the singular word
 */
function quickcab_singular_units_word($unit) {
  $units = apply_filters('quickcab_singular_units', array(
    'miles'      => esc_html__('mile', 'quickcab'),
    'kilometers' => esc_html__('kilometer', 'quickcab')
  ));

  if ( array_key_exists($unit, $units) ) {
    return $units[$unit];
  }

  return substr($units, 0, -1);
}

/**
 * Send a distancematrix request to google to get journey information
 *
 * @since  1.0
 * @param  array  $options     options of the journey
 * @return [stdClass|boolean]  the results or false on failure
 */
function quickcab_get_journey_details($options) {
  $base_url = "https://maps.googleapis.com/maps/api/distancematrix/json?";
  $units = quickcab_get_option('distance_units');

  $api_key = quickcab_get_option('google_static_maps_api_key') ? quickcab_get_option('google_static_maps_api_key') : quickcab_get_option('google_maps_api_key');

  $gmaps_options = array(
    'key'          => $api_key,
    'units'        => $units == 'miles' ? 'imperial' : 'metric',
    'origins'      => $options['origins'],
    'destinations' => $options['destinations'],
    'mode'         => 'driving',
    'language'     => get_google_maps_locale_code()
  );

  if ( array_key_exists('departure_time', $options) ) {
    $gmaps_options['departure_time'] = $options['departure_time'];
  }

  if ( array_key_exists('arrival_time', $options) ) {
    $gmaps_options['arrival_time'] = $options['arrival_time'];
  }

  $url = $base_url . http_build_query($gmaps_options);

  $ch = curl_init();
  curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, false);
  curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
  curl_setopt($ch, CURLOPT_URL, $url);
  $result = curl_exec($ch);
  curl_close($ch);

  return json_decode($result);
}

function quickcab_calculate_base_transit_buffer_times($order_id) {
  $order = new QuickCab_Order($order_id);

  $base_location = quickcab_get_option('vehicle_base_location');

  $base_to_collection_options = array(
    'origins' => $base_location,
    'destinations' => $order->get_starting_address(),
    'arrival_time' => $order->get_departure_timestamp()
  );
  $base_to_collection_journey = quickcab_get_journey_details($base_to_collection_options);
  $base_to_collection_matrix_data = $base_to_collection_journey->rows[0]->elements[0];

  $destination_to_base_options = array(
    'origins' => $order->get_destination_address(),
    'destinations' => $base_location,
    'arrival_time' => $order->get_estimated_arrival_timestamp()
  );
  $destination_to_base_journey = quickcab_get_journey_details($destination_to_base_options);
  $destination_to_base_matrix_data = $destination_to_base_journey->rows[0]->elements[0];

  $results = array(
    'base_to_collection' => null,
    'destination_to_base' => null,
  );

  if ( $base_to_collection_matrix_data && $base_to_collection_matrix_data->status !== 'ZERO_RESULTS' ) {
    $results['base_to_collection'] = $base_to_collection_matrix_data->duration->value;
  }

  if ( $destination_to_base_matrix_data && $destination_to_base_matrix_data->status !== 'ZERO_RESULTS' ) {
    $results['destination_to_base'] = $destination_to_base_matrix_data->duration->value;
  }

  return $results;
}

function quickcab_get_wordpress_timezone() {
  $timezone_string = get_option('timezone_string');

  if ( !empty($timezone_string) ) {
    return $timezone_string;
  }

  $offset = get_option('gmt_offset');
  $hours = intval($offset);
  $minutes = abs(($offset - intval($offset)) * 60);
  $offset = sprintf('%+03d:%02d', $hours, $minutes);
  return $offset;
}

function quickcab_error_notice() {
  if ( !quickcab_get_option('google_maps_api_key') ) {
    ?><div class="error notice">
      <p><?php
        echo esc_html__('QuickCab: You must save API Key 1 in the QuickCab settings.', 'quickcab');
      ?></p>
    </div><?php
  }

  if ( !quickcab_get_option('google_static_maps_api_key') ) {
    ?><div class="error notice">
      <p><?php
        echo esc_html__('QuickCab: You must save API Key 2 the QuickCab settings.', 'quickcab');
      ?></p>
    </div><?php
  }
}
add_action('admin_notices', 'quickcab_error_notice');
